# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import Plugin
from Parser import TAGS_NONE, TAGS_PANGO, TAGS_HTML

import re

ERROR =''

colorCodes = (
'ffffff','000000','00007F','009300','FF0000','7F0000','9C009C','FC7F00',
'FFFF00','00FC00','009393','00FFFF','0000FC','FF00FF','7F7F7F','D2D2D2',
'E7E6E4','CFCDD0','FFDEA4','FFAEB9','FFA8FF','B4B4FC','BAFBE5','C1FFA3',
'FAFDA2','B6B4B7','A2A0A1','F9C152','FF6D66','FF62FF','6C6CFF','68FFC3',
'8EFF67','F9FF57','858482','6E6D7B','FFA01E','F92611','FF20FF','202BFF',
'1EFFA5','60F913','FFF813','5E6464','4B494C','D98812','EB0505','DE00DE',
'0000D3','03CC88','59D80D','D4C804','333335','18171C','944E00','9B0008',
'980299','01038C','01885F','389600','9A9E15','473400','4D0000','5F0162',
'000047','06502F','1C5300','544D05')

colorCodesHex = re.compile('\[[cC]=#([0-9A-Fa-f]{6})\](.*?)\[/[cC]\]')
colorCodesCode = re.compile('\[[cC]=([0-9]{1,2})\](.*?)\[/[cC]\]')

backColorCodesHex = re.compile('\[[aA]=#([0-9A-Fa-f]{6})\](.*?)\[/[aA]\]')
backColorCodesCode = re.compile('\[[aA]=([0-9]{1,2})\](.*?)\[/[aA]\]')

#colorIrcCode = re.compile("\xc2\xb7\$([0-9]{1,2}),([0-9]{1,2})|·\$([0-9]{1,2}).*?")
colorIrcCode = re.compile("\xc2\xb7\$([0-9]{1,2})?,?([0-9]{1,2})?")
#colorIrcRGB = re.compile("\xc2\xb7\$\(([0-9]{3}),([0-9]{3}),([0-9]{3})\)")

#don't try to pack them into one regexp. please.

formatBb = {}
formatIrc = {}

tmpBb = "bius"
tmpIrc = "#&@'"

for i in range(4):
    bb = tmpBb[i]
    irc = tmpIrc[i]
    doubleCase = (bb+bb.upper(), bb+bb.upper())
    doubleIrc = (irc,irc)
    formatBb[bb] = re.compile('(\[[%s]\](.*?)\[/[%s]\])' % doubleCase)
    formatIrc[bb] = re.compile("\xc2\xb7(\%s)(.*?)(\xc2\xb7\%s|$)" % doubleIrc)
del bb, irc, doubleCase, doubleIrc

span = re.compile('(<span.*?>)|(</span>)')

removeList = (colorCodesHex, colorCodesCode, backColorCodesHex, backColorCodesCode)

getTagDict = {
    'background': ('background="#%s" ','background-color: #%s; '),
    'foreground': ('foreground="#%s" ','color: #%s; '),
    'b': ('weight="bold"', 'font-weight: bold'), 
    'u': ('underline="single"', 'text-decoration:underline'),
    'i': ('style="italic"', 'font-style: italic'),
    's': ('strikethrough="true"', 'text-decoration: line-through'),
}

#\xc2 is for utf-8..
customStatus = re.compile("^(.*)\xc2\xa0{([^}]*)}$")

def getCustomStatus(nick):
    '''parse custom status, return tuple (nick, status)'''
    result = customStatus.search(nick)
    if result:
        return result.groups()
    else:
        return (nick, '')

class MsnPlusMarkup:
    def __init__( self ):
        self.isHtml = False

    def removeMarkup( self, text ):
        '''remove the [b][/b] etc markup for pango and html markup'''
        all = removeList + tuple(formatBb.values() + formatIrc.values())
        for i in all:
            text = re.sub( i, lambda x:x.groups()[1], text)
             
        text = re.sub( colorIrcCode, '', text )

        return text
        
    def replaceMarkup( self, text ):
        '''replace the [b][/b] etc markup for pango and html markup'''

        text = text.replace('\xc2\xb7&amp;','\xc2\xb7&').replace('\xc2\xb7&quot;','\xc2\xb7"')\
        .replace('\xc2\xb7&apos;','\xc2\xb7\'') #@ROGER: i hate this, i hate it too..
        self.more = 0
        self.backgroundColor = self.frontColor = ''
                
        self.openSpan = None
        def q(dictionary, tag, text): #quick handler
            def handler(x):
                return self.getTag({tag: ''}, x.groups()[1])
            return re.sub(dictionary[tag], handler, text)

        for tag in 'bius':
            text = q(formatBb, tag, text)
            text = q(formatIrc, tag, text)

        text = re.sub( colorCodesHex, self.hexToTag, text )
        text = re.sub( colorCodesCode, self.codeToTag, text )

        text = re.sub( backColorCodesHex, self.bHexToTag, text )
        text = re.sub( backColorCodesCode, self.bCodeToTag, text )
        
        text = re.sub( colorIrcCode, self.ircCodeToTag, text ) 

        if self.openSpan != None:
            text += '</span>'
        
        return text.replace('\xc2\xb7&','\xc2\xb7&amp;').replace('\xc2\xb7"','\xc2\xb7&quot;')\
        .replace('\xc2\xb7\'','\xc2\xb7&apos;')
        
    def ircCodeToTag( self, data ):
        text = ''
        color1, color2 = data.groups()

        if self.frontColor != '' and color1 == None:
            color1 = self.frontColor

        if self.backgroundColor != '' and  color2 == None:
            color2 = self.backgroundColor
        
        self.frontColor = color1
        self.backgroundColor = color2
        
        try:
            if color1 != None and color2 != None and \
               int(color1) <= len(colorCodes) and \
               int(color1) <= len(colorCodes):
                fghex = colorCodes[int(color1)].lower()
                bghex = colorCodes[int(color2)].lower()
                text = self.getTag({'foreground': fghex, 'background': bghex},
                    None)

            elif color1 != None and int(color1) <= len(colorCodes):
                fghex = colorCodes[int(color1)].lower()
                text = self.getTag({'foreground': fghex}, None)
                        
            elif color2 != None and int(color2) <= len(colorCodes):
                bghex = colorCodes[int(color2)].lower()
                text = self.getTag({'background': bghex}, None)
        except IndexError:
            # i'm giving up. i can't deal with tuples. i fail
            pass
                    
        if self.openSpan != None:
            text = '</span>' + text

        if text:
            self.openSpan = True
            return text
        
    def hexToTag( self, data ):
        color, text = data.groups()
        return self.getTag({'foreground': color.lower()}, text)
  
    def codeToTag( self, data ):
        code, text = data.groups()
        if int(code) <= len(colorCodes):
            hex = colorCodes[int(code)].lower()
            return self.getTag({'foreground': hex}, text)
        else:
            return text
            
    def bHexToTag( self, data ):
        color, text = data.groups()
        return self.getTag({'background': color.lower()}, text) 
        
    def bCodeToTag( self, data ):
        code, text = data.groups()
        if int(code) <= len(colorCodes):
            hex = colorCodes[int(code)].lower()
            return self.getTag({'background': hex}, text)
        else:
            return text     
            
    def RGBToHTMLColor( self, rgb_tuple ):
        '''convert an (R, G, B) tuple to #RRGGBB'''
        return '#%02x%02x%02x' % rgb_tuple
       
    def HTMLColorToRGB( self, colorstring ):
        '''convert #RRGGBB to an (R, G, B) tuple'''
        colorstring = colorstring.strip()
        if colorstring.startswith('#'): 
            colorstring = colorstring[1:]
            
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)

    def getTag( self, attrdict, text ):
        attrs = []
        for key in attrdict.keys():
            if key in getTagDict:
                attr = getTagDict[key][int(self.isHtml)]
            attr = attr.replace("%s", attrdict[key])
            attrs.append(attr)

        if self.isHtml:
            tagattr = 'style="'
            for attr in attrs:
                tagattr += attr
            tagattr += '"'
        else:
            tagattr = ''
            for attr in attrs:
                tagattr += attr
        
        if text == None:
            return '<span ' + tagattr + '>'
        else:
            return '<span ' + tagattr + '>' + text + '</span>'

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        
        Plugin.Plugin.__init__( self, controller, msn )
        
        self.description = _( 'Messenser Plus like plugin for emesene' )
        self.authors = { 'Roger Duran' : 'RogerDuran at gmail dot com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _( 'Plus' )
        self.name = 'Plus'

        self.config = controller.config
        self.config.readPluginConfig(self.name)

        self.controller = controller
        
        self.plusMarkup = MsnPlusMarkup()
        self.parserFilterId = None
        
    def start( self ):
        '''start the plugin'''
        self.parserFilterId = self.controller.unifiedParser.connect( 'filter', self.plusParse )
        self.controller.emit( 'preferences-changed' )
        self.enabled = True
        
    def stop( self ):    
        '''stop the plugin'''
        self.controller.unifiedParser.disconnect( self.parserFilterId )
        self.controller.emit( 'preferences-changed' )
        self.enabled = False
        
        
    def plusParse(self, obj, parser, filterdata):
        # get a plain string
        format, objects = filterdata.serialize(filterdata.list)
        # get a msnPlusMarkup instance

        if parser and parser.tags != TAGS_NONE:
        # we have markup
            self.plusMarkup.isHtml = False
            if parser.tags == TAGS_HTML:
                # and we have html, not pango, so we tell to msnplusmarkup
                self.plusMarkup.isHtml = True

            # replace msn plus markup with pango/html
            format = self.plusMarkup.replaceMarkup(format)
        else:
            # we don't have markup
            # remove all msn plus markup
            format = self.plusMarkup.removeMarkup(format)

        # put back the objects
        filterdata.list = filterdata.deserialize(format, objects)
        
    def action( self ):
        pass
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        if ERROR != '':
            return ( False, ERROR )
            
        return ( True, 'Ok' )



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''base classes that represent abstract GUI elements to build configuration
dialog'''

class Base(object):
    '''the base class for all the items that can contain a value, it 
    contains a identifier and a value, the callback is called when the
    value changes, the callback must receive 4 arguments:
    * this object
    * the identifier
    * the value
    * a boolean that is true if the configurarion dialog is closed
    (that means that it's the last change on the element, useful if you
    dont want to update things everytime the user changes the content, and
    only when is the last change)'''

    def __init__(self, identifier, callback=None):
        '''class constructor'''
        self.identifier = identifier
        self._value = None
        self.callback = callback

    def _get_value(self):
        '''return value'''
        return self._value

    def _set_value(self, value):
        '''set value'''
        self._value = value
        self.on_value_changed()

    value = property(fget=_get_value, fset=_set_value)

    def on_value_changed(self, is_last_change=False):
        '''call the calback if not none with the arguments especified on the
        class doc, is_last_change will be True if it's the last change on the
        object (when the config dialog is closed'''
        if self.callback:
            self.callback(self, self.identifier, self.value, is_last_change)

    def validate(self, value=None):
        '''since no validations can be done on this fiels, always return
        True'''
        return (True, 'OK', self)

class Validable(Base):
    '''a class that contain methods to validate the content'''

    def __init__(self, identifier, callback=None):
        '''class constructor'''
        Base.__init__(self, identifier, callback)
        self.validators = []

    def add_validator(self, validator, error_message):
        '''add a validator method to the validators that will be called
        on the content when the validate method is called.
         if the validator return False, the error_message will be displayed'''

        self.validators.append((validator, error_message))

    def validate(self, value=None):
        '''validate the value with all the validators or self.value if
        value is None, the first that fails
        will return the error message, if some fails will return 
        (False, error_message, self) if none fails return (True, 'OK', self)'''

        for (validator, error_message) in self.validators:
            if not validator(value or self.value):
                return (False, error_message, self)

        return (True, 'OK', self)

class Text(Validable):
    '''a class that represent an abstract field that contains a label and
    a text field with an optional text content'''

    def __init__(self, identifier, label, text=None, callback=None):
        '''class constructor'''
        Validable.__init__(self, identifier, callback)
        self.label = label
        self.value = text or ''

class Password(Text):
    '''a class that represent an abstract field that contains a label and
    a password field with an optional text content'''

    def __init__(self, identifier, label, text=None, callback=None):
        '''class constructor'''
        Text.__init__(self, identifier, label, text, callback)

class CheckBox(Base):
    '''a class that represent an abstract field that contains a label and
    can be set to checked or not checked'''

    def __init__(self, identifier, label, value=False, callback=None):
        '''class constructor'''
        Base.__init__(self, identifier, callback)
        self.label = label
        self.value = value

class RadioGroup(Base):
    '''a class that represent an abstract group of fields on which only one
    can be selected, every item has a label and the group has a text that
    describe the groups, for example group_label="fruits",
    labels=("apple", "orange", "banana") selected_index=1
    the identifiers are the values that will be returned as the selecetd
    index, for example the label can be A_pple and the identidier apple,
    or the label can be translated, but identifier stay the same'''

    def __init__(self, identifier, labels, identifiers, group_label, 
            selected_index=0, callback=None):
        '''class constructor, labels is a list or tuple of strings and 
        selected_index is the index of the selected index by default, if
        the index is out of range, the first item will be selected.
        identifier is the identifier of the group, identifiers is a list or
        tuple of the identifier value for each label'''
        Base.__init__(self, identifier, callback)
        if len(labels) < 2:
            raise ValueError("labels size < 2")

        if len(labels) != len(identifiers):
            raise ValueError("number of labels and identifiers differ")

        self.labels = labels
        self.group_label = group_label
        self.selected_index = selected_index
        self.identifiers = identifiers
        
        if self.selected_index < 0 or self.selected_index > len(self.labels):
            self.selected_index = 0

class Group(object):
    '''a class that represent a logic group of elements, the way it is 
    represented can be a frame or something like that, it has a optional
    name for the group, if label is none, then no frame or label will be
    displayed on the group'''

    def __init__(self, label=None):
        '''class constructor'''
        self.label = label
        self.items = []

    def add_item(self, item):
        '''add an item to the group, the item can be any element'''
        self.items.append(item)

    def on_last_change(self):
        '''call last change for all the containing items'''
        for item in self.items:
            if issubclass(type(item), Base):
                item.on_value_changed(True)
            elif issubclass(type(item), Group):
                item.on_last_change()

    def validate(self):
        '''calidate all the containing elements and return (True, 'OK')
        if all validated and (False, error_message) of the first validation
        that failed'''

        for item in self.items:
            (validated, message, element) = item.validate()
            if not validated:
                return (validated, message, element)

        return (True, 'OK', self)


class Tab(Group):
    '''a class that represent a containter tab with elements'''

    def __init__(self, label):
        '''class constructor'''
        Group.__init__(self, label)

class TabGroup(Group):
    '''a class that represent a group of tabs'''

    def __init__(self):
        '''class constructor'''
        Group.__init__(self, None)
    
    def add_item(self, item):
        '''add an item to the group, the item can be any element'''
        if type(item) == Tab:
            Group.add_item(self, item)
        else:
            raise ValueError("item is not of type Tab")

def build(element):
    '''this method should be overrided by the implementation module, it should
    return a non modal window with an accept button, that will call to
    a method like this on close:
   
    (validated, message, element) = element.validate()

    # here a dialog should be displayed and the window not closed
    if not validated:
        dialog.error("field '" + element.identifier + \
        "' with value '" + element.value + "' not valid: \n" + message)
    else:
        # all fields are valid
        element.on_last_change()
        dialog_window.close() # or similar
    '''
    raise NotImplementedError



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import pango
import gobject
import time

from urllib import quote
from warnings import warn

import emesenelib
from emesenecommon import MAX_MESSAGE_LENGTH
import dialog
import abstract.stock as stock

import ConversationUI
import FileTransfer

class CustomEmoticons(object): # FIXME: wtf?
    def __init__(self):
        self.emoticons = {}
        
    def setNew(self, user, shortcut, id):
        if user != None: user = user.lower()
        if self.emoticons.has_key(user):
            self.emoticons[user].update({shortcut:id})
        else:
            self.emoticons.update({user:{shortcut:id}})
        
    def get(self, user=None):
        if user != None: user = user.lower()
        if self.emoticons.has_key(user):
            return self.emoticons[ user ]
        else:
            return []

class Conversation(gobject.GObject):
    '''This class is an abstraction of a conversation, it is used to separate
    the data from the GUI in a conversation to let us have single and tabbed
    windows with the same codebase (also MVC is good :P)'''

    def __init__(self, controller, switchboard):
        '''Constructor'''
        gobject.GObject.__init__(self)

        self.callbackIdList = []
        self.P2PSignals = {}
        
        self.ui = None
        self.switchboard = None
        self.setSwitchboard(switchboard)
        msn = switchboard.msn
        self.controller = controller
        self.parser = controller.unifiedParser
        self.config = controller.config
        self.title = ""
        
        self.lastMessageMail = '' # the mail of the user who sent the last message
        self.isCurrent = False # if True this is the tab that has the focus
        self.closed = False
        self.textBuffer = gtk.TextBuffer()
        self.autoreplySent = False # if true we allready sent the autoreply message
        self.inputText = ''
        self.theme = controller.theme
        self.parentConversationWindow = None
        
        msn.connect('user-attr-changed', self.onUserAttrChanged)
        msn.connect('custom-emoticon-transfered', self.onCustomEmoticonTransfered)
        
        #TODO: from now on this signals will be emitted by conversation window or conversationUI
        controller.connect('color-changed', self.onColorChanged)
        controller.connect('font-changed', self.onFontChanged)
        
        self.ui = ConversationUI.ConversationUI(self.controller, self)
        self.customEmoticons = CustomEmoticons()
        self.parser = self.controller.unifiedParser
       
        self.sendOffline = False
        self.lastSpeaker = ''
        self.user = switchboard.user
        
        self.transfers = []
    
    def onFtInvite(self, p2p, session, context, sender):
        ft = FileTransfer.FileTransfer(self.controller, p2p, self, session,
            context, sender)
        self.transfers.append(ft)
        self.ui.transfers.add(ft)
        self.ui.transfers.show_all()
    
    def sendFile(self, path):
        '''This sends a file'''
        # TODO: handle multichat
        p2p = self.controller.msn.p2p[self.switchboard.firstUser]
        sender = emesenelib.msn_p2p.FTSender(p2p, path)
        self.onFtInvite(p2p, sender.session_id, sender.context, 'Me')
    
    def getSwitchboard(self):
        return self.switchboard
        
    def close(self):
        '''close the tab'''
        self.parentConversationWindow.closeTab(self)
        self.setClosed(True)
        
    def onColorChanged(self, controller, colorStr):
        self.config.user['fontColor'] = colorStr
        self.ui.input.applyAttrsToInput()
    
    def onFontChanged(self, controller, font, bold, italic, size):
        self.setFont(font, italic, bold, size)
        
        self.ui.input.toolbar.setFontBold(bold)
        self.ui.input.toolbar.setFontItalic(italic)
        
        self.ui.input.applyAttrsToInput()
        
    def onUserAttrChanged(self, msnp, contact):
        if contact.email in self.getMembers() and \
           self.isCurrent and self.parentConversationWindow:
            
            win = self.parentConversationWindow
            win.update_title()
            
            if not self.config.user['avatarsInTaskbar']:
                win.set_icon(self.getWindowIcon())

    def getStatusIcon(self):
        '''returns the status icon for this conversation'''
        members = self.getMembers()
        theme = self.controller.theme
        user = None
        
        if len(members) > 1: 
            return theme.getImage('groupChat')
        if len(members) == 0:
            return theme.getImage('userPanel')
        
        user = self.controller.getContact(members[0])
        if not user:
            return theme.getImage('userPanel')
        return theme.statusToPixbuf(user.status)

    def getWindowIcon(self):
        '''returns the window icon for this conversation'''
        members = self.getMembers()
        user = None
        
        if len(members) == 1:
            user = self.controller.getContact(members[0])
            
        if user and self.config.user['avatarsInTaskbar'] and \
                self.controller.theme.hasUserDisplayPicture(user):
            return self.controller.theme.getUserDisplayPicture(user, 64, 64)
        return self.getStatusIcon()

    def getFontColor(self):
        '''return the user color'''
        return self.config.user['fontColor']

    def setFont(self, font, italic=False, bold=False, size=10):
        '''set the font of the user text'''
        self.config.user['fontFace'] = font
        self.config.user['fontItalic'] = italic
        self.config.user['fontBold'] = bold
        self.config.user['fontSize'] = size
    
    def getTitle(self):
        '''return a title according to the users in the conversation'''
        if not self.controller:
            return self.title

        members = self.getMembers()
        if len(members) > 1:
            self.title = _('Group chat')
        elif len(members) == 1:
            if self.config.user['useAliasIfAvailable']:
                title = self.controller.contacts.get_alias(members[0])
            if not title:
                title = self.controller.contacts.get_nick(members[0])

            if title:
                self.title = title

        return self.title

    def getUser(self):
        '''return the (local) user mail'''
        return self.switchboard.user
        
    def getRTL(self, message):
        '''check whether it's an right-to-left string'''
        try:
            if pango.find_base_dir(message, -1) == pango.DIRECTION_RTL:
                return '1'
        finally:
            return '0'

    def getStyle(self, message=''):
        '''return the style string to use in the sendMessage method'''

        effectValue = ''

        if self.config.user['fontBold']:
            effectValue += 'B'

        if self.config.user['fontItalic']:
            effectValue += 'I'

        if self.config.user['fontUnderline']:
            effectValue += 'U'
        
        if self.config.user['fontStrike']:
            effectValue += 'S'

        color = self.config.user['fontColor'].replace('#', '')
        color = color[ 4:6 ] + color[ 2:4 ] + color[ :2 ]
        
        face = self.config.user['fontFace'].replace(' ', '%20')

        return "X-MMS-IM-Format: FN=" + face + \
            "; EF=" + effectValue + "; CO=" + color + \
            "; PF=0;RL=" + self.getRTL(message)

    def getId(self):
        '''return the id of the switchboard'''
        return self.switchboard.getId()

    def getOnlineUsers(self):
        '''This method returns a list ol mails of the contacts who are not offline'''
        return self.switchboard.getOnlineUsers()

    def invite(self, mail):
        '''invite a user to the conversation'''
        self.switchboard.invite(mail)

    def getMembers(self):
        '''return a list of the members in the conversation'''
        members = self.switchboard.getMembers()
        if len(members) != 0: return members

        members = self.switchboard.getInvitedMembers()
        if len(members) != 0: return members
        
        return [self.switchboard.firstUser]

    def getWindow(self):
        '''return the window that hold this conversation'''
        return self.parentConversationWindow

    def setWindow(self, window):
        '''set the window that hold this conversation'''

        self.parentConversationWindow = window

    def setIsCurrent(self, current):
        '''set the isCurrent attribute, if true, this conversation
        is the tab that is shown'''
        self.isCurrent = current

    def getIsCurrent(self):
        '''return the value of isCurrent'''
        return self.isCurrent

    def receiveNudge(self, switchboard, mail):
        '''This method is called when a nudge is received in the
        switchboard'''
        nick = self.controller.msn.getUserDisplayName(mail)
        self.appendOutputText(None, _("%s just sent you a nudge!")% \
          self.parser.getParser(nick).get(False), 'information')
        self.doMessageWaiting(mail)

    def receiveOIM(self, nick, message, date):
        '''This method is called when a offline message is received'''
        self.appendOutputText(nick, message, 'offline_incoming', \
            timestamp=time.mktime(date))
        self.doMessageWaiting('')
        
    def receiveError(self, msnp, to, message, error):
        '''This method is called when a error message is received'''
        self.appendOutputText('Error', "Can\'t send message (%s)\n%s" % \
            (error, message), 'error')
        self.doMessageWaiting('')

    def onReceiveMessage(self, switchboard, mail, nick, message,
                         format, charset):
        '''This method is called when a message is received in the switchboard'''
        self.controller.conversationManager.emit('receive-message', self, \
            mail, nick, message, format, charset)

    def onInkMessage(self, switchboard, mail, filename):
        '''This method is called when an ink message is received
        in the switchboard'''

        self.doMessageWaiting(mail)
        self.appendOutputText(mail, quote(filename), 'ink_incoming')

    def doMessageWaiting(self, mail):
        win = self.parentConversationWindow
        if win and (not win.has_toplevel_focus() or not self.isCurrent):
            self.ui.setMessageWaiting(mail)
            self.parentConversationWindow.setUrgency()
            self.parentConversationWindow.show()
        else:
            self.ui.setDefault(mail)
        
    def do_receive_message(self, mail, nick, message, format, charset):
        '''This method is called when a message is received
        in the switchboard'''

        self.doMessageWaiting(mail)
        
        if self.config.user['autoReply'] and not self.autoreplySent:
            msg = self.config.user['autoReplyMessage']

            # no gettext here, it's a semi standard way
            # to identify automessages
            self.switchboard.sendMessage('AutoMessage: ' + msg)
            self.appendOutputText(None, 'AutoMessage: %s\n' % msg, \
                'information')
            self.autoreplySent = True
        
        if message is not None:
            self.appendOutputText(mail, message, 'incoming', \
                self.parseFormat(mail, format))

    def userJoin(self, switchboard, mail):
        '''This method is called when someone joins the conversation'''
        if switchboard.isGroupChat():
            nick = self.controller.msn.getUserDisplayName(mail)
            nick = self.parser.getParser(nick).get(False)
            self.appendOutputText('', \
                _("%s has joined the conversation") % nick, 'information')
       
        if self.isCurrent:
            self.parentConversationWindow.update_title()
            self.parentConversationWindow.set_icon(self.getWindowIcon())
        
        if self.ui:
            self.ui.update()

    def userLeave(self, switchboard, mail):
        '''method called when someone leaves the conversation'''
        nick = self.controller.msn.getUserDisplayName(mail)
        nick = self.parser.getParser(nick).get(False)
        self.appendOutputText('', _("%s has left the conversation")%nick, \
            'information')
        
        if self.isCurrent:
            self.parentConversationWindow.update_title()
            self.parentConversationWindow.set_icon(self.getWindowIcon())
        
        if self.ui:
            self.ui.update()
        
    def sbStatusChange(self, switchboard):
        if self.ui:
            self.ui.update()

    def inviteUser(self, mail):
        '''method called when the user selects a friend in the invite dialog'''
        if self.switchboard.status == 'closed':
            self.reconnect()

        self.invite(mail) 
        self.ui.messageWaiting[mail] = False
        self.ui.contactTyping[mail] = False
        self.ui.update()

    def doNudge(self):
        '''this method is called when the user clicks the nudge button'''
        if self.switchboard.status == 'closed':
            self.reconnect()

        try:
            self.switchboard.sendNudge()
        except Exception:
            self.reconnect()
            self.switchboard.sendNudge()

        self.appendOutputText(None, _("you have sent a nudge!"), 'information')

    def reconnect(self):
        '''reconnect the switchboard'''
        if not self.controller or not self.controller.msn:
            return

        user = self.getMembers()[0]
        self.setSwitchboard(self.controller.msn.getSwitchboard(user))
        self.autoreplySent = False

    def splitMessage(self, message):
        '''Split large messages'''
        messageChunks = []
        messageLen = len(message)
        msgStart = 0
        while msgStart < messageLen:
            chunk = message[msgStart:msgStart+MAX_MESSAGE_LENGTH]
            chunkLen = len(chunk)
            if chunkLen == MAX_MESSAGE_LENGTH:
                chunkEnd = chunk.rfind(' ')
                if chunkEnd!=-1 and chunkEnd>0:
                    messageChunks.append(chunk[0:chunkEnd])
                    msgStart += chunkEnd+1
                else:
                    msgStart += chunkLen
                    messageChunks.append(chunk)
            else:
                msgStart += chunkLen
                messageChunks.append(chunk)
        return messageChunks

    def do_send_message(self, message, retry=0):
        '''Send the message from the UI input. This chooses between OIM and
        switchboard to send the message.'''
        
        remoteMail = self.switchboard.firstUser
        remoteStatus = self.controller.contacts.get_status(remoteMail)

        def do_send_offline(response, mail='', message=''):
            '''callback for the confirm dialog asking to send offline 
            message'''
            if response == stock.YES:
                self.sendOffline = True
                self.switchboard.msn.msnOIM.send(mail, message)
                self.appendOutputText(self.user, message, 'outgoing')
        
        if self.switchboard.status == 'error' and remoteStatus == 'FLN':
            
            if self.sendOffline == True:
                do_send_offline(stock.YES, remoteMail, message)
            else:
                dialog.yes_no(
                 _("Are you sure you want to send a offline message to %s") %\
                 remoteMail, do_send_offline, remoteMail, message)

            return
            
        if self.switchboard.status == 'closed':
            self.reconnect()

        messageChunks = self.splitMessage(message)
        for chunk in messageChunks:
            try:
                self.switchboard.sendCustomEmoticons(chunk)
                self.switchboard.sendMessage(chunk, self.getStyle(chunk))
                self.appendOutputText(self.user, chunk, 'outgoing')
            except Exception, e:
                raise
                print str(e)
                self.reconnect()
                if retry < 3:
                    self.do_send_message(
                        ''.join(messageChunks[messageChunks.index(chunk):]), \
                        retry + 1)
                else:
                    self.appendOutputText(None, _('Can\'t send message'), \
                        'information')
                return

    def sendMessage(self, message):
        '''send a message to the conversation'''
        self.controller.conversationManager.emit('send-message', self, message)
            
    def sendIsTyping(self):
        '''an easy method to send the is typing message'''
        if self.switchboard.status == 'closed':
            self.reconnect()

        try:
            self.switchboard.sendIsTyping()
        except Exception:
            self.reconnect()
            self.sendIsTyping()

    def parseFormat(self, mail, format):
        '''parse the format of a mail and return the style'''

        # if the useFriendsUnifiedFormat flag is set, then return that format

        if self.config.user['useFriendsUnifiedFormat']:
            font = self.config.user['friendsUnifiedFont']
            color = self.config.user['friendsUnifiedColor']
            return 'font-family: ' + emesenelib.common.escape(font) + \
                ';color: ' + emesenelib.common.escape(color) + ';'

        # FN=Sans; EF=; CO=000000; PF=0
     
        style = ''
     
        if format.find("FN=") != -1:
            font = format.split('FN=')[1].split(';')[0].replace('%20', ' ')
            style += 'font-family: ' + emesenelib.common.escape(font) + ';'

        if format.find("CO=") != -1:
            color = format.split('CO=')[1].split(';')[0]
                 
            if len(color) == 3:
                color = color[2] + color[1] + color[0]
                style += 'color: #' + emesenelib.common.escape(color) + ';'
            else:
                color = color.zfill(6)
                
            if len(color) == 6:
                color = color[4:6] + color[2:4] + color[:2]
                style += 'color: #' + emesenelib.common.escape(color) + ';'

        if format.find("EF=") != -1:
            effect = set(format.split('EF=')[1].split(';')[0])

            if "B" in effect: style += 'font-weight: bold;'
            if "I" in effect: style += 'font-style: italic;'
            if "U" in effect: style += 'text-decoration: underline;'
            if "S" in effect: style += 'text-decoration: line-through;'
        
        return style

    def setSwitchboard(self, switchboard):
        '''set a new switchboard for the conversation.
        useful if the conversation is closed
        and the other user starts a new one'''

        signalDict = {
            'nudge': self.receiveNudge,
            'message': self.onReceiveMessage,
            'ink-message': self.onInkMessage,
            'user-join': self.userJoin,
            'user-leave': self.userLeave,
            'typing': self.receiveTyping,
            'custom-emoticon-received': self.onCustomEmoticonReceived,
        }
        
        if self.switchboard:
            # disconnect old stuff
            
            while len(self.callbackIdList) > 0:
                self.switchboard.disconnect(self.callbackIdList.pop())
            
            if self.switchboard.msn:
                for mail in self.P2PSignals.keys():
                    self.switchboard.msn.p2p[mail].disconnect(\
                        self.P2PSignals.pop(mail))

        if switchboard is None:
            # the function was called just to disconnect everything
            return

        # connect new switchboard
        self.switchboard = switchboard

        for signalName in signalDict.keys():
            self.callbackIdList.append(self.switchboard.connect(signalName, \
                signalDict[signalName]))
        
        for mail in switchboard.getMembers():
            self.P2PSignals[mail] = switchboard.msn.p2p[mail].connect(\
                'file-transfer-invite', self.onFtInvite)
        
        self.autoreplySent = False

    def onCustomEmoticonReceived(self, switchboard, shortcut, msnobj):
        '''call when a smiley is received'''
        self.customEmoticons.setNew(msnobj.creator, shortcut, msnobj.sha1d)
        
    def onCustomEmoticonTransfered(self, switchboard, to, msnobj, path):
        '''call when a smiley is transfered'''
        if self.ui:
            self.ui.textview.setCustomObject(msnobj.sha1d, path, \
                type='application/x-emesene-emoticon')

    def appendOutputText(self, username, text, type, style = None, timestamp = None):
        '''append the given text to the outputBuffer'''
        
        if type.startswith('ink_'):
            type = type[4:]
            ink = True
        else:
            ink = False
        
        if type != 'incoming' and type != 'outgoing':
            self.lastSpeaker = ''
        elif username == self.lastSpeaker:
            type = 'consecutive_' + type
        
        self.lastSpeaker = username
        
        if username == self.switchboard.user:
            nick = emesenelib.common.escape(self.controller.msn.nick)
        elif username != None:
            nick = emesenelib.common.escape( \
                self.controller.msn.getUserDisplayName(username))
        else:
            nick = ''

        if timestamp is None:
            timestamp = time.time()

        displayedText = self.controller.conversationLayoutManager.layout(\
            username, text, style, self, type, timestamp, ink)

        try:
            self.ui.textview.display_html(displayedText.encode('ascii', \
                'xmlcharrefreplace'))
        except Exception, e:
            print 'error trying to display "' + displayedText + '"' 
            print e

        self.ui.scrollToBottom()              

    def getStatus(self):
        return self.switchboard.status

    def setStatus(self, value):
        self.switchboard.setStatus(value)

    def isClosed(self):
        return self.closed

    def setClosed(self, value):
        self.closed = value
        if value == True:
            self.switchboard.leaveChat()
            self.setSwitchboard(None)
    
    def receiveTyping(self, switchboard, mail):
        '''This method is called when a is typing message is received in the
        switchboard'''
        if self.ui:
            self.ui.setTyping(mail)

    def getMembersDict(self):
        '''return a dict with email as key and contact instance as value'''
        
        userDict = {}
        for mail in self.getMembers():
            contact = self.controller.getContact(mail)
            if contact:
                userDict[mail] = contact
        
        return userDict
        
    def getTextTag(self):
        '''return a text tag from the current style'''
        
        tag = gtk.TextTag()
        
        if self.config.user['disableFormat']:
            return tag
        
        tag.set_property('font', self.config.user['fontFace'])
        tag.set_property('size-points', self.config.user['fontSize'])
        tag.set_property("foreground" , self.config.user['fontColor'])
        
        if self.config.user['fontBold']:
            tag.set_property("weight" , pango.WEIGHT_BOLD)
        
        if self.config.user['fontItalic']:
            tag.set_property("style" , pango.STYLE_ITALIC)
        
        if self.config.user['fontUnderline']:
            tag.set_property("underline-set" , True)
            tag.set_property("underline" , pango.UNDERLINE_SINGLE)
        else:
            #XXX: wtf is that underline-set=True?
            tag.set_property("underline-set" , True)
            tag.set_property("underline" , pango.UNDERLINE_NONE)
            
        tag.set_property("strikethrough" , self.config.user['fontStrike'])
               
        return tag
        
gobject.type_register(Conversation)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gobject
import urllib

import SoapManager
import ContactData
import XmlTemplates

import XmlParser

import common

class ProfileManager(gobject.GObject):
    '''this class has all the methods to
    modify the contacts, groups, nick and stuff
    through SOAP, its made to make Msnp
    more modular, also this class can be
    changed for other implementation later'''

    __gsignals__ = {
        'user-attr-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        # olgGroup, newGroup
        'group-attr-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # email, id, nick, pm, status, alias, blocked
        'contact-added' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT,
            gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT,
            gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT,
            gobject.TYPE_PYOBJECT,)),
        # email
        'contact-removed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        # this signal is emited when WE change an attribute on the user
        # the only two attributes that WE can change are alias, and block
        # email, attr_name (block, alias), value
        'contact-attr-changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT, 
                gobject.TYPE_PYOBJECT,)),
        # name, id
        'group-added' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # name
        'group-removed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,)),
        # old_name, new_name
        'group-renamed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # contact, group
        'contact-added-to-group' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # contact, group
        'contact-removed-from-group' : (gobject.SIGNAL_RUN_LAST, 
            gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
    }

    def __init__(self):
        gobject.GObject.__init__(self)
        
    def getMembershipListSync(self):
        '''request the membership list dont return until we get it'''

        common.debug('request Membership list')

        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/AddressBook/FindMembership', \
            'contacts.msn.com', 443, '/abservice/SharingService.asmx', \
            XmlTemplates.membershipList)

        SoapManager.put(request)
        response = SoapManager.get()

        return self.onGetMembershipList(response)

    def getMembershipListAsync(self):
        '''request the membership list return and put a callback'''

        common.debug('request Membership list')

        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/AddressBook/FindMembership', \
            'contacts.msn.com', 443, '/abservice/SharingService.asmx', \
            XmlTemplates.membershipList, self.onGetMembershipList)

        SoapManager.put(request)

    def onGetMembershipList(self, response):
        '''method called when we receive the membership list'''

        common.debug(str(response.status))

        if response.status[0] == 200:
            self.setMembershipListXml(response.body )
            self.newCacheFile(self.user + '_ml.xml', response.body)
            #self.emit('user-list-change')
            
            return True
        else:
            return False

    def getAddressBookSync(self):
        '''request the address book return and put a callback'''
    
        common.debug('request Address book')

        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/AddressBook/ABFindAll', \
            'contacts.msn.com', 443, \
            '/abservice/abservice.asmx', XmlTemplates.addressBook)

        SoapManager.put(request)
        response = SoapManager.get()

        common.debug(str(response.status))

        if response.status[0] == 200:
            self.newCacheFile(self.user + '_ab.xml', response.body)

    def getAddressBookAsync(self):
        '''request the address book dont return until we get it'''
        
        common.debug('request Address book')

        request = SoapManager.SoapRequest(
            'http://www.msn.com/webservices/AddressBook/ABFindAll', \
            'contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.addressBook, self.onGetAddressBook)

        SoapManager.put(request)

    def onGetAddressBook(self, response):
        common.debug(str(response.status))

        if response.status[0] == 200:
            self.newCacheFile(self.user + '_ab.xml', response.body)

    def getDynamicItemsSync(self):
        '''request dynamic items (spaces and all that) dont return
        until we get that'''

        common.debug('request Dynamic Items')

        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/AddressBook/ABFindAll', \
            'contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.dynamicItems)

        SoapManager.put(request)
        response = SoapManager.get()

        return self.onGetDynamicItems(response)

    def getDynamicItemsAsync(self):
        '''request dynamic items (spaces and all that) return
        and put a callback'''

        common.debug('request Dynamic Items')

        request = SoapManager.SoapRequest(
            'http://www.msn.com/webservices/AddressBook/ABFindAll', \
            'contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.dynamicItems, self.onGetDynamicItems)

        SoapManager.put(request)

    def onGetDynamicItems(self, response):
        common.debug(str(response.status))

        if response.status[0] == 200:
            self.newCacheFile(self.user + '_di.xml', response.body)

            self.setDynamicItemsXml(response.body)
            self.getNickFromDynamicItems(response.body)
            self.changeNick(self.nick)
            self.emit('user-list-change')
            
            return True
        else:
            return False

    def getStorageAuthCache(self, email):
        common.debug('request storage auth cache ' + email)
        
        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/storage/w10/GetItemVersion', \
            'storage.msn.com', 80, '/storageservice/schematizedstore.asmx', \
            XmlTemplates.schematizedStore%(email), self.onGetStorageAuthCache)

        SoapManager.put(request)

    def onGetStorageAuthCache(self, response):
        common.debug(str(response.status) + "\n" + response.body)

    def getUserProfile(self, email, storageAuthCache = None):
        if storageAuthCache == None:
            self.getStorageAuthCache(email)
        else:
            common.debug('request user profile ' + email)

            cid = str(self.contactManager.getContact(email).cid)
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/spaces/v1/GetXmlFeed', \
                'cid-580B3F2402D4CDA4.cc.services.spaces.live.com', 80, \
                '/contactcard/contactcardservice.asmx', \
                XmlTemplates.space % (self.t, "", cid, storageAuthCache, "en-US"), \
                self.onGetUserProfile)

            SoapManager.put(request)
        
    def onGetUserProfile(self, response):
        common.debug(str(response.status) + "\n" + response.body)

    def getNickFromDynamicItems(self, dynamicItems):
        # get the nick from the XML
        try:
            temp = dynamicItems.split('<contactType>Me</contactType>')\
                    [1].split('</displayName>')[0]
            nick = common.unescape(temp.split('<displayName>')[1])
            
        except:
            nick = self.user
        
        if nick != self.nick:
            oldNick = self.nick
            self.nick = nick
            self.emit('self-nick-changed', oldNick, self.nick)

    def addUser(self, email, group):
        '''add an user to the friend list'''

        if self.contactManager.contact_exists(email):
            # We already have that contact
            return

        self.sendDL('ADL', email, '1')
        self.sendDL('ADL', email, '2')

        # need to add it in prevention of ILN. Don't update user list
        self.contactManager.addNewContact(email)
        
        request = SoapManager.SoapRequest(
            'http://www.msn.com/webservices/AddressBook/ABContactAdd', \
            'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.contactAdd % (email,), \
            self.onUserAdded, (email, group))

        SoapManager.put(request)

    def onUserAdded(self, response, email, group=''):
        '''this method is called when the addUser soapRequest get a response'''

        common.debug('add user: ' + email + ' ' + str(response.status))
        
        if response.status[0] == 200:
            # Now that we have the contact id, we show it
            self.contactManager.setContactIdXml(email, response.body)
            guid = response.body.split('<guid>')[1].split('</guid>')[0]
            self.emit('contact-added', email, guid, None, None, 'FLN', None,
                False)

            if group == '':
                self.emit('user-list-change')
            else:
                # once we now the id we can add it to a group
                self.addUserToGroup(email, group)
        else:
            self.contactManager.removeContact(email)
            self.emit('user-list-change')
            self.emit('error', 'user-add-error', 
                _('User could not be added: %s') % \
                common.parseSoapFault(response))

    def removeUserFromGroup(self, user, group):
        '''remove user from a group'''

        contactID = self.contactManager.getContactId(user)
        sourceGid = self.contactManager.getGroupId(group)

        if sourceGid == '' or sourceGid == 'nogroup':
            return
        
        self.contactManager.removeUserFromGroup(user, sourceGid)
        self.emit('user-list-change')

        request = SoapManager.SoapRequest(
            'http://www.msn.com/webservices/AddressBook/ABGroupContactDelete', \
            'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.deleteUserFromGroup % (contactID,sourceGid), \
            self.onUserRemovedFromGroup, (sourceGid, group, user))

        SoapManager.put(request)


    def onUserRemovedFromGroup(self, response, groupId, group, user):
        common.debug('remove user from group: ' + str(response.status))

        if response.status[0] == 200:
            self.emit('contact-removed-from-group', user, group)
        else:
            self.contactManager.addUserToGroup(user, groupId)
            self.emit('user-list-change')
            self.emit('error', 'user-remove-error', _('User could not be remove: %s') % common.parseSoapFault(response))

    def addUserToGroup(self, user, group):
        '''add a user to a group'''

        gid = self.contactManager.getGroupId(group)
        contactID = self.contactManager.getContactId(user)
        if gid == None:
            common.debug('Group not found')
            return
        if gid in self.contactManager.getContact(user).groups:
            common.debug('User already in group')
            return
        if gid == 'nogroup':
            common.debug('Cannot move to no group')
            return            
        
        self.contactManager.addUserToGroup(user, gid)
        self.emit('user-list-change')

        request = SoapManager.SoapRequest(
            'http://www.msn.com/webservices/AddressBook/ABGroupContactAdd', \
            'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.moveUserToGroup % (gid, contactID), \
            self.onUserAddedToGroup, (user, group, gid))

        SoapManager.put(request)

    def onUserAddedToGroup(self, response, user, group, groupId):
        common.debug('add user to group: ' + str(response.status))

        if response.status[0] == 200:
            self.emit('contact-added-to-group', user, group)
        else:
            self.contactManager.removeUserFromGroup(user, groupId)
            self.emit('user-list-change')
            self.emit('error', 'user-add-to-group-error', \
                _('User could not be added to group: %s') % \
                common.parseSoapFault(response))

    def moveUserToGroup(self, user, srcGroup, destGroup, stage=0):
        '''move a user from a group erasing it from de source group'''

        if stage == 0:
            contactID = self.contactManager.getContactId(user)
            sourceGid = self.contactManager.getGroupId(srcGroup)
            destGid = self.contactManager.getGroupId(destGroup)
            
            # moving to/from No group = adding to destGroup / removing from srcGroup
            if sourceGid == 'nogroup':
                self.addUserToGroup(user, destGroup)
                return
            if destGid == 'nogroup':
                self.removeUserFromGroup(user, srcGroup)
                return
            
            # check whether or not it's an allowed movement
            if srcGroup == destGroup:
                common.debug('src and dest groups are the same')
                return
            elif self.contactManager.getGroup(destGid).getUser(user) != None:
                common.debug('dest group already contain the user')
                return
            
            # make the visual changes
            self.contactManager.removeUserFromGroup(user, sourceGid)
            self.contactManager.addUserToGroup(user, destGid)
            self.emit('user-list-change')
    
            # remove the user from the srcGroup
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/ABGroupContactDelete', \
                'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                XmlTemplates.deleteUserFromGroup%(contactID,sourceGid), \
                self.onMoveUserToGroup, (user, srcGroup, destGroup, 0))
            SoapManager.put(request)
        
        elif stage == 1:
            gid = self.contactManager.getGroupId(destGroup)
            contactID = self.contactManager.getContactId(user)
        
            # add the user to the destGroup
            request = SoapManager.SoapRequest(\
                'http://www.msn.com/webservices/AddressBook/ABGroupContactAdd', \
                'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                XmlTemplates.moveUserToGroup%(gid, contactID), \
                self.onMoveUserToGroup, (user, srcGroup, destGroup, 1))
            SoapManager.put(request)
            
    def onMoveUserToGroup(self, response, user, srcGroup, destGroup, stage):
        
        common.debug('move user (stage ' + str(stage) + '): ' \
                           + str(response.status))
        
        status = response.status
        
        if response.status[0] == 200:
            if stage == 0: # continue the moving procedure
                self.moveUserToGroup(user, srcGroup, destGroup, 1)
        else:
            # restore the old visual
            sourceGid = self.contactManager.getGroupId(srcGroup)
            destGid = self.contactManager.getGroupId(destGroup)
            
            self.contactManager.removeUserFromGroup(user, destGid)
            self.contactManager.addUserToGroup(user, sourceGid)
            self.emit('user-list-change')
            self.emit('error', 'user-move-to-group-error', _('User could not be moved to group: %s') % common.parseSoapFault(response))

    def removeUser(self, email):
        '''remove an user from the friendr list'''

        self.sendDL('RML', email, '1')
        
        contact = self.contactManager.getContact(email)
        contactID = self.contactManager.getContactId(email)
        
        # make the contact not visible
        self.contactManager.removeContact(email)
        self.emit('user-list-change')

        request = SoapManager.SoapRequest(\
            'http://www.msn.com/webservices/AddressBook/ABContactDelete', \
            'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
            XmlTemplates.contactRemove % (contactID,), \
            self.onUserRemoved, (email, contact))
        SoapManager.put(request)

    def onUserRemoved(self, response, email, contact):
        '''this method is called when the removeUser soapRequest get a response
        renes means if we have to renew the di,ml,ab data'''

        common.debug('remove user: ' + email + ' ' + str(response.status))
        
        if response.status[0] == 200:
            self.emit('contact-removed', email)
        else:
            # TODO: probably need to send an ADL or sth since
            # the removal has failed
            self.contactManager.addContact(contact)
            self.emit('user-list-change')
                
            self.emit('error', 'user-remove-error', _('User could not be removed: %s') % common.parseSoapFault(response))

    def blockUser(self, email, stage=0):
        '''block an user'''

        if stage == 0:
            self.sendDL('RML', email, '2')
            self.sendDL('ADL', email, '4')
    
            self.contactManager.blockContact(email)
            contact = self.contactManager.getContact(email)
            self.emit('user-attr-changed', contact)
            
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/DeleteMember', \
                'omega.contacts.msn.com', 443, '/abservice/SharingService.asmx', \
                XmlTemplates.deleteMember%('BlockUnblock', 'Allow', email), \
                self.onUserBlocked, (email, 0))
            SoapManager.put(request)
            
        elif stage == 1:
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/AddMember', \
                'omega.contacts.msn.com', 443, '/abservice/SharingService.asmx', \
                XmlTemplates.addMember%('BlockUnblock', 'Block', email), \
                self.onUserBlocked, (email, 1))
            SoapManager.put(request)

    def onUserBlocked(self, response, email, stage):
        '''this method is called when the blockUser soapRequest get a response'''

        common.debug('block user (stage ' + str(stage) + '): ' \
                           + str(response.status))

        if response.status[0] == 200:
            if stage == 0: # continue the blocking process
                self.blockUser(email, 1)

            self.emit('contact-attr-changed', email, 'block', True)
        else:
            self.unblockUser(email, 0)
            self.contactManager.unblockContact(email)
            contact = self.contactManager.getContact(email)
            self.emit('user-attr-changed', contact)

    def unblockUser(self, email, stage=0):
        '''unblock an user'''

        if stage == 0:
            self.sendDL('RML', email, '4')
            self.sendDL('ADL', email, '2')
    
            self.contactManager.unblockContact(email)
            contact = self.contactManager.getContact(email)
            self.emit('user-attr-changed', contact)
    
            request = SoapManager.SoapRequest(\
                'http://www.msn.com/webservices/AddressBook/DeleteMember', \
                'omega.contacts.msn.com', 443, '/abservice/SharingService.asmx', \
                XmlTemplates.deleteMember%('BlockUnblock', 'Block', email), \
                self.onUserUnblocked, (email, 0))
    
            SoapManager.put(request)
        
        elif stage == 1:
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/AddMember', \
                'omega.contacts.msn.com', 443, '/abservice/SharingService.asmx', \
                XmlTemplates.addMember%('BlockUnblock', 'Allow', email), \
                self.onUserUnblocked, (email, 1))
            SoapManager.put(request)

    def onUserUnblocked(self, response, email, stage):
        '''this method is called when the unblockUser soapRequest get a response'''

        common.debug('unblock user (stage ' + str(stage) + '): ' \
                           + str(response.status))
        
        if response.status[0] == 200:
            if stage == 0:
                self.unblockUser(email, 1)
            
            self.emit('contact-attr-changed', email, 'block', False)
        else:
            self.contactManager.blockContact(email)
            contact = self.contactManager.getContact(email)
            self.emit('user-attr-changed', contact)

    def addGroup(self, group):
        '''add a group to the group list'''

        if self.contactManager.getGroupId(group) != None:
            common.debug('Unable to add: Group "' + group \
                               + '" already exists')
            return

        name = group.replace(' ', '%20')

        request = SoapManager.SoapRequest(
                    'http://www.msn.com/webservices/AddressBook/ABGroupAdd', \
                    'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                    XmlTemplates.addGroup % (group,), \
                    self.onGroupAdded, (group,))
        SoapManager.put(request)

    def onGroupAdded(self, response, group):
        '''this method is called when the addGroup soap request get a response'''

        common.debug('add group ' + str(response.status))

        if response.status[0] == 200:
            try:
                gid = response.body.split('<guid>')[1].split('</guid>')[0]
                self.contactManager.addGroup(group, gid)
                self.emit('group-added', group, gid)
                self.emit('user-list-change')
            except IndexError, e:
                common.debug('cannot add group to userlist')
                common.debug(str(e))
        else:
            self.emit('error', 'group-add-error', _('Group could not be added: %s') % common.parseSoapFault(response))

    def removeGroup(self, group):
        '''remove a group from the group list'''

        gid = self.contactManager.getGroupId(group)
        if gid:
            groupObj = self.contactManager.getGroup(gid)
            self.contactManager.removeGroup(gid)
            self.emit('user-list-change')
            
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/ABGroupDelete', \
                'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                XmlTemplates.deleteGroup % (gid,), \
                self.onGroupRemoved, (gid, groupObj))

            SoapManager.put(request)
        else:
            common.debug('Unable to remove: Group "' + group \
                               + '" does not exist')

    def onGroupRemoved(self, response, gid, group):
        '''this method is called when the removeGroup soap request get a response'''

        common.debug('remove group ' + str(response.status))
        
        if response.status[0] == 200:
            self.emit('group-removed', group)
        else:
            # TODO: change it to setGroup when it manages contacts
            self.contactManager.setGroup(gid, group)
            self.emit('user-list-change')
            self.emit('error', 'group-remove-error', _('Group could not be removed: %s') % common.parseSoapFault(response))

    def renameGroup(self, oldGroup, newGroup):
        '''rename a group from the group list'''

        if oldGroup == newGroup:
            common.debug('oldgroup and new group are the same')
            return
        if self.contactManager.getGroupId(newGroup) != None:
            common.debug('That group name is already in use')
            return
            
        gid = self.contactManager.getGroupId(oldGroup)
        if gid == None:
            common.debug('The specified group does not exist')
            return
        else:
            self.contactManager.renameGroup(gid, newGroup)
            group = self.contactManager.getGroup(gid)
            objOldGroup = ContactData.Group(oldGroup)
            self.emit('group-attr-changed', objOldGroup, group)
            request = SoapManager.SoapRequest(
                'http://www.msn.com/webservices/AddressBook/ABGroupUpdate', \
                'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                XmlTemplates.renameGroup % (gid, common.escape(newGroup)), \
                self.onGroupRenamed, (oldGroup, newGroup))
            SoapManager.put(request)

    def onGroupRenamed(self, response, oldGroup, newGroup):
        '''this method is called when the renameGroup soap request get a response'''

        common.debug('rename group ' + str(response.status))
        
        if response.status[0] == 200:
            self.emit('group-renamed', oldGroup, newGroup)
        else:
            gid = self.contactManager.getGroupId(newGroup)
            self.contactManager.renameGroup(gid, oldGroup)
            group = self.contactManager.getGroup(gid)
            # its old because we revert the changes
            objOldGroup = ContactData.Group(newGroup)
            self.emit('group-attr-changed', objOldGroup, group)
            self.emit('error', 'group-rename-error', _('Group could not be renamed: %s') % common.parseSoapFault(response))

    def changeNick(self, nick, user='Me', initial=False):
        '''change the nick, type can be Me or Normal'''
        nick = nick.decode('utf-8', 'replace').encode('utf-8')
        if user == 'Me':
            if not initial and self.nick == nick:
                common.debug('trying to set the same nick')
                return

            if len(nick) > 129:
                # to avoid problems with utf-8
                return
            
            oldNick = self.nick
            self.nick = nick
            #self.contactManager.setContactNick(self.user, self.nick)
            self.emit('self-nick-changed', oldNick, self.nick)
            
            self.socket.sendCommand("PRP", "MFN " + urllib.quote(nick))

            nick = common.escape(nick)
            if not initial:
                request = SoapManager.SoapRequest(
                    'http://www.msn.com/webservices/AddressBook/ABContactUpdate', \
                    'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                    XmlTemplates.changeNick % (user, nick), \
                    self.onNickChanged, (self.user, oldNick))
        else:
            # why this change the contact alias?
            # it's a good question, but remember to sign your
            # comments with --~~~~, like this: --dx [talk] 9/11 3:47
            oldAlias = self.contactManager.getContactAlias(user)
            
            self.contactManager.setContactAlias(user, nick)
            contact = self.contactManager.getContact(user)
            self.emit('user-attr-changed', contact)

            contactID = str(self.contactManager.getContactId(user))
            nick = str(common.escape(nick))
            if not initial:
                request = SoapManager.SoapRequest(
                    'http://www.msn.com/webservices/AddressBook/ABContactUpdate', \
                    'omega.contacts.msn.com', 443, '/abservice/abservice.asmx', \
                    str(XmlTemplates.renameContact % (contactID, nick)), \
                    self.onNickChanged, (user, oldAlias))

        if not initial:
            SoapManager.put(request)

    def onNickChanged(self, response, user, oldNick):
        '''this method is called when the changeNick soap request get a response'''
        common.debug('change nick ' + str(response.status))
        
        if response.status[0] != 200:
            if user == 'Me':
                #self.contactManager.setContactNick(self.user, oldNick)
                self.emit('self-nick-changed', self.nick, oldNick)
            else:
                self.emit('contact-attr-changed', user, 'alias', oldNick)
                self.contactManager.setContactAlias(user, oldNick)
                contact = self.contactManager.getContact(user)
                self.emit('user-attr-changed', contact)
                
            self.emit('error', 'nick-change-error', _('Nick could not be changed: %s') % common.parseSoapFault(response))

    def updateUUX(self):
        '''update personal message and current media'''
        pm = self.personalMessage
        cm = self.currentMedia
        pm = pm.decode('utf-8', 'replace').encode('utf-8')
        cm = cm.decode('utf-8', 'replace').encode('utf-8')

        self.socket.sendPayloadCommand('UUX', '', \
            '<Data><PSM>' + common.escape(pm) + '</PSM>' + \
            '<CurrentMedia>' + common.escape(cm) + '</CurrentMedia>' + \
            '<MachineGuid></MachineGuid></Data>')
        
    def changePersonalMessage(self, pm):
        '''change the personal message'''
        if self.personalMessage != pm:
            self.personalMessage = pm
            self.updateUUX()
            self.emit('self-personal-message-changed', self.user, pm)
        else:
            common.debug("duplicate pm")
        
    def changeCurrentMedia(self, cm, dict=None):
        '''change the current media'''
        if self.currentMedia != cm:
            self.currentMedia = cm
            self.updateUUX()
            self.emit('self-current-media-changed', self.user, cm, dict)
        else:
            common.debug("duplicate cm")

    def getUserDisplayName(self, mail):
        '''return the user display name or just the mail if it cant be found'''
        mail = mail.lower()
        if mail == self.user:
            return self.nick
        else:
            alias = self.contactManager.getContactAlias(mail)
                
            if alias:
                return alias
            else:
                return self.contactManager.getContactNick(mail)
    
    def setDynamicItemsXml(self, xml):
        '''modify the structure with a new DynamicItems xml'''
        
        contacts = {}
        
        for (mail, contact) in self.contactManager.contacts.iteritems():
            contacts[mail] = contact

        self.contactManager.groups = {}
        self.contactManager.contacts = {}
        
        #doc = minidom.parseString(xml)
        
        dinamicItems = XmlParser.DynamicParser(xml)
        # Retrieve groups
        for i in dinamicItems.groups:
            groupId = i['groupId']
            name = i['name']

            if groupId not in self.contactManager.groups:
                self.contactManager.setGroup(groupId , 
                    ContactData.Group(name, groupId))
                self.emit('group-added', name, groupId)
        
        # Retrieve contacts
        for i in dinamicItems.contacts:
            if 'isMessengerUser' in i and 'passportName' in i and \
               i['isMessengerUser'] == 'true':
                # valid
                email = i['passportName'].lower()
                contact = ContactData.Contact(email)
            else:
                continue

            try:
                contactId = i['contactId']
                cid = i['CID']
                contact.id = contactId
                contact.cid = cid
                
                groups = []
               
                for guid in i['groupIds']:
                    groups.append(guid)

                contact.groups = groups
                
                for j in i['Annotations']:
                    try:
                        if j['Name'] == 'AB.NickName':
                            alias = j['Value']
                            contact.alias = urllib.unquote(alias)
                    except:
                        pass
                
                displayName = i['displayName']
                contact.nick = urllib.unquote(displayName)
                
                isMobileIMEnabled = i['isMobileIMEnabled']
                contact.mobile = isMobileIMEnabled == 'true'
                
                hasSpace = i['hasSpace']
                contact.space = hasSpace == 'true'
            except KeyError:
                continue

            if email in contacts:
                contact.status = contacts[email].status
                contact.nick = contacts[email].nick
                contact.personalMessage = contacts[email].personalMessage
                contact.msnobj = contacts[email].msnobj
            
            # finally adds the contact
            self.contactManager.addContact(contact)
            self.emit('contact-added', contact.email, contact.id, contact.nick,
                contact.personalMessage, contact.status, contact.alias,
                contact.blocked)

            for group_id in contact.groups:
                self.emit('contact-added-to-group', contact.email, 
                    self.contactManager.getGroup(group_id).name)
            
        self.contactManager.updateMemberships()

    def setMembershipListXml(self, xml):
        '''modify the structure with a new MembershipList xml
        if it is the first xml that you send, send first the dynamic items'''
        
        self.contactManager.lists['Allow'] = []
        self.contactManager.lists['Block'] = []
        self.contactManager.lists['Reverse'] = []
        self.contactManager.lists['Pending'] = []
        
        #ml = minidom.parseString(xml)
        ml = XmlParser.MembershipParser(xml)

        for i in ml.memberships:
            memberRole = i['MemberRole']

            for j in i['Members']:
                try:
                    email = j['PassportName'].lower()
                    if email not in self.contactManager.lists[memberRole]:
                        self.contactManager.lists[memberRole].append(email)
                    else:
                        pass
                    
                    if memberRole == 'Pending' and 'DisplayName' in j: 
                        self.contactManager.pendingNicks[email] = \
                            j['DisplayName']

                except Exception, e:
                    pass


gobject.type_register(ProfileManager)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import time

import emesenelib.common
import Plugin

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Contructor'''
        
        Plugin.Plugin.__init__(self, controller, msn)
        
        self.description = _('Get Last something from the logs')
        self.authors = { 'Luis Mariano Guerra' : 
            'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene.org'
        self.displayName = 'LastStuff'
        self.name = 'LastStuff'
        self.controller = controller
        self.Slash = controller.Slash

    def start(self):
        '''start the plugin'''
        self.Slash.register('last', self.get_last, 'Get last stuff')
        self.enabled = True

    def get_last(self, slash_action):
        params = slash_action.getParams()
        if params:
            data = params.split(' ')
        else:
            data = ''

        logger = self.controller.pluginManager.getPlugin("Logger")

        if not logger or not logger.enabled:
            slash_action.outputText('Logger plugin not available', False)
            return

        if len(data) != 3:
            slash_action.outputText('Usage: /last # stuff account', False)
            return
        
        (stamp, stuff, account) = data
        
        try:
            num = int(data[0])
        except:
            slash_action.outputText(_('invalid number as first parameter'), 
                False)
            return
        
        if data[1] == "nick":
            results = logger.get_last_nick(account, num)
            results.reverse()
            for (stamp, result) in results:
                slash_action.outputText(result, False)
            
            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
        elif data[1] == "pm":
            results = logger.get_last_personal_message(account, num)
            results.reverse()
            for (stamp, result) in results:
                slash_action.outputText(result, False)
            
            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
        elif data[1] == "ce":
            results = logger.get_last_custom_emoticon(account, num)
            results.reverse()
            for (stamp, result) in results:
                slash_action.outputText(result, False)
            
            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
        elif data[1] == "dp":
            results = logger.get_last_display_picture(account, num)
            results.reverse()
            for (stamp, result) in results:
                slash_action.outputText(result, False)
            
            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
        elif data[1] == "message":
            results = logger.get_last_message(account, num)
            results.reverse()
            for (stamp, result) in results:
                try:
                    slash_action.outputText(result.split("\r\n")[2], False)
                except IndexError:
                    print 'malformed message on LastStuff'
            
            if len(results) == 0:
                slash_action.outputText('Empty result', False)
                
        elif data[1] == "conversation":
            results = logger.get_last_conversation(account, num)
            results.reverse()
            for (stamp, account, result) in results:
                try:
                    slash_action.outputText(
                        account + ': ' + result.split("\r\n")[2], False)
                except IndexError:
                    print 'malformed message on LastStuff'

            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
        elif data[1] == "status":
            results = logger.get_last_status(account, num)
            results.reverse()
            for (stamp, result) in results:
                slash_action.outputText('%s since %s' % (
                    emesenelib.common.reverse_status[result], 
                    time.ctime(float(stamp)))
                    , False)
            
            if len(results) == 0:
                slash_action.outputText(_('Empty result'), False)
                
    def stop(self):    
        '''stop the plugin'''
        self.Slash.unregister('last')
        self.enabled = False
        
    def check(self):
        plugin = self.controller.pluginManager.getPlugin("Logger")
        if plugin is None or not plugin.enabled:
            return (False, _('Enable the logger plugin'))

        return (True, 'Ok')




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import Plugin, Theme

import gtk
import gobject
import pango

from time import *


class MainClass( Plugin.Plugin ):
    
    def __init__( self, controller, msn ):
        Plugin.Plugin.__init__( self, controller, msn )
        self.description = 'Show a list with the history of online/offline events of every contact with a timestamp.'
        self.authors = { 'mr.archano' : 'archano@gmail.com', 'Jan de Mooij' : 'jandemooij@gmail.com' }
        self.website = ''
        self.displayName = 'StatusHistory'
        self.name = 'StatusHistory'
        self.enabled = False
        self.controller = controller
        
        self.config = controller.config
        self.config.readPluginConfig(self.name)

        self.showStatusImage = ((self.config.getPluginValue(self.name, 'showStatusImage', '1')) == '1')
                
    def start( self ):
        self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.box = gtk.VBox()
        self.addComboBox(self.box)
        
        self.addEntry('connected')

        self.box.show_all()
        self.controller.mainWindow.vbox.pack_start(self.box, False, False)
        self.onOnlineId = self.controller.msn.connect('user-online', self.on_online)
        self.onOfflineId = self.controller.msn.connect('user-offline', self.on_offline)
        self.enabled = True
    
    def stop( self ):
        self.controller.mainWindow.vbox.remove( self.box )
        self.disconnect( self.onOnlineId )
        self.disconnect( self.onOfflineId )
        self.enabled = False
    
    def check( self ):
        return ( True, 'Ok' )
    
    def configure( self ):
        l=[]
        l.append( Plugin.Option( 'showStatusImage', bool, _('Show status image:'), _('Show status image:'), self.showStatusImage))
        
        response = Plugin.ConfigWindow( _( 'StatusHistory plugin config' ), l ).run()
        if response != None:
            self.showStatusImage = response['showStatusImage'].value
            self.config.setPluginValue( self.name, 'showStatusImage', str(int(self.showStatusImage)) )

            if self.enabled:
                self.box.remove(self.comboBox)
                self.addComboBox(self.box)
        return True
    
    ####################
    
    def addComboBox(self, box):
        self.comboBox = gtk.ComboBox(self.listStore)
        
        self.timeTextCell = gtk.CellRendererText()
        self.nickTextCell = gtk.CellRendererText()
        self.nickTextCell.set_property('ellipsize', pango.ELLIPSIZE_END)

        if self.showStatusImage:
            self.statusCell = gtk.CellRendererPixbuf()
        else:
            self.statusCell = gtk.CellRendererText()

        self.comboBox.pack_start(self.timeTextCell, False)
        self.comboBox.pack_start(self.nickTextCell, True)
        self.comboBox.pack_start(self.statusCell, False)
        
        self.comboBox.add_attribute(self.timeTextCell, 'text', 0)
        self.comboBox.add_attribute(self.nickTextCell, 'text', 1)

        if self.showStatusImage:
            self.comboBox.set_cell_data_func(self.statusCell, self.cellLayoutFunc)
        else:
            self.comboBox.add_attribute(self.statusCell, 'text', 2)

        self.comboBox.set_active(0)        
        self.comboBox.show()
        box.pack_start(self.comboBox, False, False)
    
    def cellLayoutFunc(self, layout, cell, model, iter):
        '''show pixbuf for status'''
        item = model[iter][2]
        
        if item == None: 
            return
        if item == 'connected': 
            item = 'online'
        pixbuf = self.controller.theme.statusToPixbuf(item)
        cell.set_property('pixbuf', Theme.resizePixbuf(pixbuf, 16, 16))
    
    def addEntry(self, status, email = None):
        '''add a new entry to the liststore'''
        if email != None:
            nick = self.controller.msn.contactManager.getContactNick(email)
        else:
            nick = self.controller.msn.user
        
        time = strftime('[%H:%M:%S]', localtime())
        self.listStore.prepend([time, nick, status])
        self.comboBox.set_active(0)
    
    def on_online( self, msnp, email, oldStatus ):
        if oldStatus == 'FLN':
            self.addEntry('online', email)
            
    def on_offline( self, msnp, email ):
        self.addEntry('offline', email)
        



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import gtk

class GroupMenu(gtk.Menu):
    '''This class represent the popup menu that is displayed when you right click a group on
    the userList'''

    def __init__(self, controller, group):
        gtk.Menu.__init__(self)

        self.controller = controller

        if group.id != 'nogroup':
            renameGroupMenuItem = self.newImageMenuItem\
                        (_('Re_name group...'), gtk.STOCK_EDIT)
            self.add(renameGroupMenuItem)
            deleteGroupMenuItem = self.newImageMenuItem\
                        (_('Re_move group'), gtk.STOCK_DELETE)
            self.add(deleteGroupMenuItem)

            self.add(gtk.SeparatorMenuItem())

            deleteGroupMenuItem.connect('activate', self.on_delete_group_activate)
            renameGroupMenuItem.connect('activate', self.on_rename_group_activate)

        addUserGroupMenuItem = self.newImageMenuItem\
                    (_('_Add contact...'), gtk.STOCK_ADD)
        self.add(addUserGroupMenuItem)
        addGroupMenuItem = self.newImageMenuItem\
                    (_('Add _group...'), gtk.STOCK_ADD)
        self.add(addGroupMenuItem)

        addUserGroupMenuItem.connect('activate', self.on_add_user_group_activate)
        addGroupMenuItem.connect('activate', self.on_add_group_activate)

        self.show_all()

        self.group = group

    def newImageMenuItem(self, label, stock = None, img = None):
        mi = gtk.ImageMenuItem(_(label))

        if stock != None:
            mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
        elif img != None:
            image = gtk.Image()
            image.set_from_pixbuf(img)
            mi.set_image(image)
        return mi

    def on_add_user_group_activate(self, *args):
        self.controller.addUserDialog(self.group.name)

    def on_add_group_activate(self, *args):
        self.controller.groups.add_dialog()

    def on_delete_group_activate(self, *args):
        self.controller.groups.remove(self.group.name)

    def on_rename_group_activate(self, *args):
        self.controller.groups.rename_dialog(self.group.name)




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import struct
import urllib
import md5

import Msnobj
import Switchboard

import common

#------------------------- FROM HERE -----------------------------
# Copyright 2005 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details
_PRODUCT_KEY = 'O4BG@C7BWLYQX?5G'
_PRODUCT_ID = 'PROD01065C%ZFN6F' 
MSNP11_MAGIC_NUM = 0x0E79A9C1

def doChallenge(chlData):
    md5digest = md5.md5(chlData + _PRODUCT_KEY).digest()

    # Make array of md5 string ints
    md5Ints = struct.unpack("<llll", md5digest)
    md5Ints = [(x & 0x7fffffff) for x in md5Ints]

    # Make array of chl string ints
    chlData += _PRODUCT_ID
    amount = 8 - len(chlData) % 8
    chlData += "".zfill(amount)
    chlInts = struct.unpack("<%di" % (len(chlData)/4), chlData)

    # Make the key
    high = 0
    low = 0
    i = 0
    while i < len(chlInts) - 1:
        temp = chlInts[i]
        temp = (MSNP11_MAGIC_NUM * temp) % 0x7FFFFFFF
        temp += high
        temp = md5Ints[0] * temp + md5Ints[1]
        temp = temp % 0x7FFFFFFF

        high = chlInts[i + 1]
        high = (high + temp) % 0x7FFFFFFF
        high = md5Ints[2] * high + md5Ints[3]
        high = high % 0x7FFFFFFF

        low = low + high + temp

        i += 2

    high = littleEndify((high + md5Ints[1]) % 0x7FFFFFFF)
    low = littleEndify((low + md5Ints[3]) % 0x7FFFFFFF)
    key = (high << 32L) + low
    key = littleEndify(key, "Q")

    longs = [x for x in struct.unpack(">QQ", md5digest)]
    longs = [littleEndify(x, "Q") for x in longs]
    longs = [x ^ key for x in longs]
    longs = [littleEndify(abs(x), "Q") for x in longs]
    
    out = ""
    for x in longs:
        x = hex(long(x))
        x = x[2:-1]
        x = x.zfill(16)
        out += x.lower()
    
    return out

def littleEndify(num, c="L"):
    return struct.unpack(">" + c, struct.pack("<" + c, num))[0]
# ------------------------------ TO HERE ----------------------------------

class SignalHandler( object ):
    '''This class represent the default signal handler for core
    you should inherit from this class, if you know what you are doing
    you can create your own base class without inheriting from this one'''
    
    def __init__( self, msn ):
        '''contructor'''
        
        self.msn = msn
        self.socket = self.msn.socket
        
    def challenge( self, msnp, command, tid, params ):
        '''reply a challenge made from the server'''
        
        out = doChallenge( params )
        self.socket.sendPayloadCommand( 'QRY' , _PRODUCT_ID , out )
                      
    def statusChange( self, msnp, command, tid, params ):
        '''handle a status change'''
        global flag
        #ILN 9 IDL xxxxxxxxxxx@yahoo.com .%20J%20i%20N%20G%20. 1342554148 %3Cmsnobj%20Creator%3D%22xxxxxxxxxxxxx%40yahoo.it%22%20Size%3D%225637%22%20Type%3D%223%22%20Location%3D%22TFR10B.dat%22%20Friendly%3D%22AAA%3D%22%20SHA1D%3D%22X0qXqwshIMyNnCGFYDEmOrLfFrE%3D%22%20SHA1C%3D%22CTsCEDGiY82DrNMYGVJAj01x%2F%2BA%3D%22%2F%3E

        t = params.split( " " )
        status = t[ 0 ]
        email = t[ 1 ].lower()
        nick = urllib.unquote(t[2])
        nick = nick.decode('utf-8', 'replace').encode('utf-8')
        
        oldStatus = self.msn.contactManager.getContactStatus( email )
        self.msn.contactManager.setContactStatus( email, status )
        self.msn.contactManager.setContactNick( email, nick )
        
        if len( t ) == 5:
            msnobj = t[ 4 ]
            contact = self.msn.contactManager.getContact( email )
            if contact != None:
                msnObject = Msnobj.createFromString( msnobj )

                if contact.msnobj == None or \
                   (msnObject != None and msnObject != contact.msnobj):
                    
                    contact.msnobj = msnObject
                    self.msn.emit( 'msnobj-changed', msnObject, (oldStatus=='FLN') )     
        
        if self.msn.contactManager.getContactNick( email ) != nick:
            self.msn.emit( 'nick-changed', email, nick )     
       
        if self.msn.contactManager.getContactStatus(email) != status:
            self.msn.emit('contact-status-change', email, status)   
        
        contact = self.msn.contactManager.getContact(email)
        self.msn.emit('user-attr-changed', contact)
    
    def statusOnline( self, msnp, command, tid, params ):
        '''handle a friend going online'''
        
        status = tid
        t = params.split(' ')
        email = t[ 0 ].lower()
        
        nick = urllib.unquote(t[1])
        nick = nick.decode('utf-8', 'replace').encode('utf-8')
        
        oldStatus = self.msn.contactManager.getContactStatus( email )
	
        self.msn.contactManager.setContactStatus( email, status )

        if len( t ) == 4:
            msnobj = t[ 3 ]
            contact = self.msn.contactManager.getContact( email )
            if contact != None:
                msnObject = Msnobj.createFromString( msnobj )
                
                if contact.msnobj == None or \
                   (msnObject != None and \
                    msnObject != contact.msnobj):
                    contact.msnobj = msnObject
                    self.msn.emit( 'msnobj-changed', msnObject, 
                                    (oldStatus=='FLN') )

        if self.msn.contactManager.getContactNick( email ) != nick:
            self.msn.emit( 'nick-changed', email, nick )
            
        self.msn.contactManager.setContactNick( email, nick )
        
        if oldStatus != status:
            common.debug('%s (%s) changed status from %s to %s' % \
            (nick, email, oldStatus, status), 'signal handler')
            self.msn.emit( 'user-online', email, oldStatus )
            self.msn.emit( 'contact-status-change', email, status )
        
        contact = self.msn.contactManager.getContact( email )
        self.msn.emit( 'user-attr-changed', contact )   
    
    def statusOffline( self, msnp, command, tid, params ):
        '''handle a friend going offline'''
        
        email = tid.lower()
        self.msn.contactManager.setContactStatus( email, 'FLN' )
        common.debug('%s disconnected' % (email,), 'signal handler')
        self.msn.emit( 'user-offline', email )
        contact = self.msn.contactManager.getContact( email )
        self.msn.emit( 'user-attr-changed', contact )
        self.msn.emit('contact-status-change', email, 'FLN')
    
    def switchboardInvitation( self, msnp, command, tid, params ):
        '''handle the switchboard invitation, this is the first message
        that a buddy send to you when he wants to chat with you'''
        #example:
        # RNG 146980 207.46.2.154:1863 CKI 1138138395.4459 
        # example@passport.com Example%20Name U messenger.hotmail.com
        
        completeCommand = command + ' ' + tid + ' ' + params
        if len( completeCommand.split( ' ' ) ) != 9:
            print 'invalid message in switchboardInvitation, not opening switchboard'
            return
        
        ( command, sessionID, host, authenticationType, 
        authenticationString, user, userName, u, 
        server ) = completeCommand.split( ' ' )

        user = user.lower()
        sb = Switchboard.Switchboard( tid, msnp, 'invited' )
        msnp.on_new_switchboard(sb)
        sb.setConnectionString( completeCommand )
        
    def serverMessage( self, msnp, command, tid, params, payload ):
        '''called when we receive a MSG in the main socket'''
        payloadData = {}
        for line in payload.split('\r\n'):
            try:
                key, value = line.split(': ',1)
                payloadData.update({key:value})
                common.debug(line)
            except:
                pass

        if payloadData.has_key('Content-Type') == True:
            ct = payloadData['Content-Type'].split('; ')[0]
            if( ct == 'text/x-msmsgsinitialemailnotification' ):
                # Initial Mail Notification
                self.msn.inboxUnreadMessages = int(
                    payloadData['Inbox-Unread'] )
                self.msn.otherMessagesUnreaded=int(
                    payloadData['Folders-Unread'] )
                
                self.msn.emit( 'initial-mail-notification' )
                    
            elif( ct == 'text/x-msmsgsinitialmdatanotification' ):
                #Initial Offline Messages Notification
                self.msn.msnOIM.parseMailData( payloadData['Mail-Data'] )
                try:
                    self.msn.inboxMessages=int(
                        payload.split('<I>')[1].split('</I>')[0])
                except: pass    
                try:
                    self.msn.inboxUnreadMessages=int(
                        payload.split('<IU>')[1].split('</IU>')[0])
                except: pass
                try:
                    self.msn.otherMessages=int(
                        payload.split('<O>')[1].split('</O>')[0])
                except: pass
                try:
                    self.msn.otherMessagesUnreaded=int(
                        payload.split('<OU>')[1].split('</OU>')[0])
                except: pass
                    
                self.msn.emit( 'initial-mail-notification' )
                
                if self.msn.msnOIM.getMessageCount() > 0:
                    self.msn.emit( 'offline-message-waiting', 
                        self.msn.msnOIM )
                    
            elif( ct == 'text/x-msmsgsemailnotification' ):
                # New Email Notification
                From = payloadData['From']
                FromAddr = payloadData['From-Addr']
                Subject = payloadData['Subject']
                MessageURL = payloadData['Message-URL']
                PostURL = payloadData['Post-URL']
                id = payloadData['id']
                
                if payloadData['Dest-Folder'] == 'ACTIVE':
                    self.msn.inboxMessages += 1
                    self.msn.inboxUnreadMessages +=1
                else:
                    self.msn.otherMessages += 1
                    self.msn.otherMessagesUnreaded +=1  

                self.msn.emit( 'new-mail-notification', 
                    From, FromAddr, Subject, MessageURL , PostURL, id )
            elif( ct == 'text/x-msmsgsactivemailnotification' ):
                # Email Movement Notification
                src = payloadData['Src-Folder']
                dest = payloadData['Dest-Folder']
                delta = int( payloadData['Message-Delta'] )
                if dest == 'trAsH':
                    if src == 'ACTIVE':
                        self.msn.inboxMessages -= delta
                        self.msn.inboxUnreadMessages -=delta
                    else:
                        self.msn.otherMessages -= delta
                        self.msn.otherMessagesUnreaded -=delta
                elif dest == 'ACTIVE':
                    if src == 'ACTIVE':
                        self.msn.inboxUnreadMessages -= delta
                    elif src != 'trAsH':
                        self.msn.otherMessages -= delta
                        self.msn.otherMessagesUnreaded -=delta
                        self.msn.inboxMessages += delta
                        self.msn.inboxUnreadMessages +=delta
                    else:
                        self.msn.inboxMessages += delta
                        self.msn.inboxUnreadMessages +=delta
                                
                else:
                    if src == 'ACTIVE':
                        self.msn.inboxMessages -= delta
                        self.msn.inboxUnreadMessages -=delta
                        self.msn.otherMessages += delta
                        self.msn.otherMessagesUnreaded +=delta
                    elif src == dest:
                        self.msn.otherMessagesUnreaded -=delta
                    else:
                        self.msn.otherMessages += delta
                        self.msn.otherMessagesUnreaded +=delta
                            
                self.msn.emit( 'mail-movement-notification' )
            elif( ct == 'text/x-msmsgsoimnotification' ):
                #offline message notiffication
                self.msn.msnOIM.parseMailData( payloadData['Mail-Data'] )
                if self.msn.msnOIM.getMessageCount() > 0:
                    self.msn.emit( 'offline-message-waiting', 
                        self.msn.msnOIM )
            
            elif ct == 'text/x-msmsgsprofile':
               msnp.parse_demographics(payload) 
            else:
                print 'Unhandled Content Type:', ct
        else:
            print 'Unhandled MSG, printing Raw Data:'
            print payload



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import weakref
import emesenelib.common
from emesenecommon import PATH

class PluginManager( object ):
    '''This module handles the plugins'''
    
    def __init__( self, controller ):
        '''Constructor'''

        # we do the import here because we need to wait until the
        # config creates the plugin directory on the user home
        userPlugins = {}
        if os.name == 'posix':
            sys.path.append(PATH.CONFIG_DIR)
            userPlugins = self.inspectPackage(PATH.CONFIG_DIR, 
                    PATH.PLUGINS_HOME)
            
        systemPlugins = self.inspectPackage(PATH.APP_PATH, 
                PATH.PLUGINS_SYSTEM_WIDE)
        
        # system plugins have preference over user plugins
        # unless user plug's version is greater then sys' one
        for i in userPlugins.keys():
            if not i in systemPlugins:
                # the plugin is only in user's place
                continue
            # both user and system have the plugin
            elif not userPlugins[i]:
                # user version is None, so it can't be less
                del userPlugins[i]
            elif self.comparePluginVersions(userPlugins[i], 
                                             systemPlugins[i] ):
                # user version is greater than system version
                del systemPlugins[i]
            else:
                del userPlugins[i]
        
        self.userPlugins = userPlugins.keys()
        self.systemPlugins = systemPlugins.keys()
        
        self.controller = controller
        
        # a dict that contain an instance of the object plugin (because the 
        # module can have multiple clases)
        self.plugin = {}
        
        for i in self.getModules():
            self.loadPlugin( i )
        
        self.runningPlugins = []
        
    def inspectPackage(self, path, package):
        '''Searches the given path for plugins in package.
        Returns a dict with the found plugins and their versions, if any.'''
                
        try:
            __import__(package, globals(), locals(), [])
        except:
            print 'LO QUE!!!'
        
        path = path + PATH.DIR_SEP + package
        modules = [ x.split( '.' )[ 0 ] for x in os.listdir( path ) \
                    if x.endswith( '.py' ) and not x.startswith( '_' ) \
                    and x != 'Plugin.py' ]
        plugins = {}
        plugins = plugins.fromkeys( modules )
        
        for i in modules:
            try:
                mod = __import__( package + '.' + i, globals(), \
                                  None, [ 'VERSION' ] )
                plugins[i] = getattr( mod, 'VERSION' )
            except AttributeError, error:
                pass
            except Exception, e:
                del plugins[i]
                print('Exception importing %s\n%s'
                % (i, str(e)), 'PluginManager')
        
        return plugins
    
    def comparePluginVersions( self, v1, v2 ):
        '''Compare versions of the format 'x.y.z'
        Returns True if v1 and v2 can be compared and v1 > v2, otherwise False'''
        
        v1 = v1.strip().split( '.' )
        v2 = v2.strip().split( '.' )
        
        try:
            for i in range( min( len( v1 ), len( v2 ) ) ):
                if int( v1[ i ] ) > int( v2[ i ] ):
                    return True
                if int( v1[ i ] ) < int( v2[ i ] ):
                    return False
        except:
            return False
        
        if len( v1 ) > len( v2 ):
            return True
        else:
            return False
            
    
    def getModules(self):
        '''Returns a list with the plugins full module names  on the form:
        module_name.plugin_name'''
        user = [ PATH.PLUGINS_HOME + '.' + x for x in self.userPlugins ]
        system = [ PATH.PLUGINS_SYSTEM_WIDE + '.' + x for x in self.systemPlugins ]
        user.extend( system )
        return user
    
    def getPlugins(self):
        '''Returns a list with the plugins names'''
        
        list = self.userPlugins[:]    # makes a copy of userPlugins
        list.extend( self.systemPlugins )
        return list

    def pluginToModuleName(self, plugin):
        '''Converts a plugin name into a module name with full path'''
        if plugin in self.systemPlugins:
            return PATH.PLUGINS_SYSTEM_WIDE + '.' + plugin
        elif plugin in self.userPlugins:
            return PATH.PLUGINS_HOME + '.' + plugin
        else:
            return ''
        
    def loadPlugin( self, module, doReload=False ):
        '''Instanciate an object of the plugin, the possible exceptions should
        be handled by the caller.'''

        weakmsn = weakref.ref(self.controller.msn)

        if module in self.getModules():
            try:
                mod = __import__( module, globals(), locals(), [] )
                name = module.split('.')[ 1 ]
                plugin = getattr( mod, name )
                if doReload:
                    reload(plugin)

                self.plugin[ name ] = plugin.MainClass( self.controller, \
                    weakmsn() )
            except Exception, e:
                print 'Plugin ' + module + ' could not be initialized'
                print 'Reason: ' + str( e  )

    def checkPlugin( self, name ):
        '''check if the plugin can be initialized'''
        
        if self.plugin.has_key( name ):
            return self.plugin[ name ].check()
        else:
            return [ False, 'The plugin isn\'t loaded' ]
        
    def startPlugin( self, name ):
        '''start the plugin, call check before this!'''
        
        if self.plugin.has_key( name ):
            self.plugin[ name ].start()
            self.runningPlugins.append( name )
            return True
        else:
            return False
        
    def stopPlugin( self, name ):
        '''stop the plugin'''
        
        if self.plugin.has_key( name ):
            if name in self.runningPlugins:
                self.plugin[ name ].stop()
                self.runningPlugins.remove( name )
                return True
        else:
            return False
        
    def getPlugin( self, name ):
        '''return the plugin if exist or None'''
        
        if self.plugin.has_key( name ):
            return self.plugin[ name ]
        else:
            return None
        
    def destroy( self ):
        '''Stops all the running plugins and cleans references'''
        
        for i in self.runningPlugins:
            try:
                self.plugin[ i ].stop()
            except:
                emesenelib.common.debug( 'Plugin ' + self.plugin[ i ].name + \
                    ' failed to stop' )
        



# -*- coding: utf-8 -*-

#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys
import shutil
import gobject
import emesenelib.common
from emesenecommon import PATH

DEFAULT_GLOBAL_CONFIG = {
    'rememberMe': True,
    'rememberMyPassword': False,
    'debug': False,
    'binary': False,
    'httpMethod': False,
    'useProxy': False,
    'proxyHost': 'http://127.0.0.1',
    'proxyPort': 80,
    'mainWindowWidth': 260,
    'mainWindowHeight': 440,
    'mainWindowX': -1,
    'mainWindowY': -1,
    'autoLogin': False,
    'lastLoggedAccount': '',
    'rgbaColormap' : False,
    'overrideDesktop' : '',
}


DEFAULT_USER_CONFIG = {
    'theme': 'default',
    'smilieTheme': 'default',
    'conversationLayout' : 'default',
    'smallIcons': False,
    'showByNick': True,
    'showOffline': False,
    'showEmptyGroups': False,
    'orderByStatus': False,
    'collapsedGroups': '',
    'autoReply': False,
    'autoReplyMessage': 'Im not here (this is an autoreply)',
    'rememberMyPassword': False,
    'fontFace': 'Sans',
    'fontColor': '#000000',
    'fontSize': 10,
    'fontUnderline' : False,
    'fontBold' : False,
    'fontItalic' : False,
    'fontStrike' : False,
    'showTimestamp': False, # XXX: check if this works
    'convWindowWidth': 450,
    'convWindowHeight': 480,
    'convInputHeight' : 140,
    'saveLogsAutomatically' : False,
    'activePlugins': 'PersonalMessage', # CSV of plugin names
    'emoticonDir': '',
    'avatarDir' : '',
    'avatarPath' : '',
    'parseSmilies' : True,
    'typingColor': '#339999',
    'messageWaitingColor': '#ef2929',
    'personalMessageColor': '#AAAAAA',
    'useFriendsUnifiedFormat' : False,
    'friendsUnifiedFont' : 'Sans',
    'friendsUnifiedColor' : '#000000',
    'windows' : False,
    
    # almost every setting in this block has signals
    'showUserPanel' : True,
    'showSearchEntry' : True,
    'showStatusCombo' : True,
    'showMenubar' : True,
    'showMenu' : True,
    'showHeader' : True,
    'showToolbar' : True,
    'showAvatars' : True,
    'showStatusBar' : True,
    'showAvatarMine' : True, # signal not needed
    'showAvatarOther' : True, # signal not needed
    'showTabCloseButton' : True, # TODO: w/o signal
    'avatarsInUserList' : True, # TODO: w/o signal
    'avatarsInTaskbar' : False,
    'showMailTyping' : True,
    'showSendButton' : False,

    'showCountContact' : False,
    'showTabImageStatus' : False,
    'mediaEnabled' : False,
    'blinkTrayIcon' : True,
    'statusbarHighLight' : True,
    'sortNickGroupByContact' : 1, # 0 = sort by nickname | 1 = sort by mail
    'sortNickGroupByStatusPriority' : 0, # sort contact in group by status

    'statusTransformation' : 'corner|alpha', # valid values are corner, pixelate, alpha(seperated by | )

    'useAliasIfAvailable' : True, # show alias everywhere if an alias is set
    'sendTyping' : True, # send the is typing notification to other clients
    'useFancyLabel' : False, 
    'userListAvatarSize': 32,
    
    'receivedFilesSortedByUser' : False,
    'receivedFilesDir' : '~/',
    'disableEsc' : False, # disable the esc key to close conversations
    'disableFormat': False,
}

class Config(gobject.GObject):
    '''This class contains all the stuff relative to the configuration of
    emesene. You can get or set configuration values and will be saved to
    a file for persistence'''
     
    __gsignals__ = {
        # value, oldvalue
        'change' : (gobject.SIGNAL_RUN_LAST | gobject.SIGNAL_DETAILED, \
          gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
    }


    # -- GLOBAL CONFIG -- #
    def __init__(self):
        '''Constructor : this will create needed files for global configuration'''
        gobject.GObject.__init__(self)
        
        self.currentUser = ''
        self.glob = {}
        self.user = {}
        
        # This dict is a bit different from the two others
        # It's not a key : value dict but a plugin_name : [ key : value ] dict
        # Also it's a plain old dict, don't use it directly
        self.pluginsConfigDict = {}
        
        if os.name != 'nt' or hasattr(sys, "frozen"):
            # home dirs don't work outside py2exe (!)
            _mkdir('', PATH.HOME_DIR, '.config')
            _mkdir('Config dir', PATH.CONFIG_DIR)
            _mkdir('Theme dir in config', PATH.THEME_HOME_PATH)
            _mkdir('Smilies dir in config', PATH.SMILIES_HOME_PATH)
            _mkdir('Convthemes dir in config', PATH.CONVTHEMES_HOME_PATH)
            _mkdir('Theme dir in config', PATH.THEME_HOME_PATH)
            
            if _mkdir('Plugins dir in config', PATH.PLUGIN_HOME_PATH):
                shutil.copyfile(PATH.PLUGIN_SYSTEM_WIDE_PATH + '/__init__.py', \
                                PATH.PLUGIN_HOME_PATH + '/__init__.py')
            
        if _mkfile('Config file', PATH.CONFIG_DIR, 'config'):
            self.glob = ConfigDict(self, self.writeGlobalConfig, \
                DEFAULT_GLOBAL_CONFIG)
            self.writeGlobalConfig()
            
        self.readGlobalConfig()
        
    def readGlobalConfig(self):
        '''read the config file and create a dictionarie with key and value
        of all the key=value\n in the config file'''
        
        globalConfigDict = DEFAULT_GLOBAL_CONFIG.copy()
        conf = None
        try:
            conf = open(PATH.CONFIG_DIR + '/config', 'r')
            string = conf.read()
            
            for i in string.splitlines():
                if i != '':
                    try:
                        delim = i.find('=')
                        key = i[:delim]
                        value = i[delim+1:]
                        if key in DEFAULT_GLOBAL_CONFIG:
                            globalConfigDict[ key ] = value
                        else:
                            emesenelib.common.debug(key + ' is not a valid config key, ignored')
                    except Exception, e:
                        emesenelib.common.debug(key + ' config value is incorrect')
                                    
            conf.close()
        except:
            if conf:
                conf.close()

        self.glob = ConfigDict(self, self.writeGlobalConfig, DEFAULT_GLOBAL_CONFIG, globalConfigDict)
            
    def writeGlobalConfig(self):
        '''write the config to the file, overwrite current config file'''
        
        try:
            conf = open(PATH.CONFIG_DIR + '/config', 'w')
        
            for k, v in self.glob:
                if type(v) == bool:
                    conf.write(k + '=' + str(int(v)) + '\n')
                else:
                    conf.write(k + '=' + str(v) + '\n')
                    
            conf.close()
        except Exception, e:
            emesenelib.common.debug('exception writing config:\n')
            emesenelib.common.debug(e)
                
    # -- USER CONFIG -- #
    def setCurrentUser(self, email):
        ''' Create and/or read needed file for user config
        /!\ This function MUST be called before any set/getUserConfig'''
        
        self.currentUser = email.replace('@', '_').replace('.', '_')
        self.glob['lastLoggedAccount'] = email
            
        
        if email != '':
             
            _mkdir('User config dir', PATH.CONFIG_DIR, self.currentUser)

            if _mkfile('Config file created: ', PATH.CONFIG_DIR, \
               self.currentUser, 'config'):
                self.user = ConfigDict(self, self.writeUserConfig, \
                    DEFAULT_USER_CONFIG)
                self.writeUserConfig()
            
            _mkdir('', PATH.CONFIG_DIR, self.currentUser, 'logs')
            _mkdir('', PATH.CONFIG_DIR, self.currentUser, 'cache')
            _mkdir('', PATH.CONFIG_DIR, self.currentUser, 'avatars')
            _mkdir('', PATH.CONFIG_DIR, self.currentUser, 'custom_emoticons')
            self.readUserConfig()

        else:
            self.user = {}
      
    def getCurrentUser(self):
        return self.currentUser
        
    def readUserConfig(self):
        '''read the config file and create a dictionarie with key and value
        of all the key=value\n in the config file'''
        
        userConfigDict = DEFAULT_USER_CONFIG.copy()
        conf = None
        try:
            conf = open(PATH.CONFIG_DIR + '/' + self.currentUser + '/config', 'r')
            string = conf.read()
                        
            for i in string.splitlines():
                if i != '':
                    try:
                        delim = i.find('=')
                        key = i[:delim]
                        value = i[delim+1:]
                        if key in DEFAULT_USER_CONFIG:
                            userConfigDict[ key ] = value
                        else:
                            emesenelib.common.debug(key + ' is not a valid config key, ignored')
                    except Exception, e:
                        emesenelib.common.debug(key + ' config value is incorrect')
                    
            conf.close()
        except:
            if conf:
                conf.close()

        self.user = ConfigDict(self, self.writeUserConfig, DEFAULT_USER_CONFIG, userConfigDict)
            
    def writeUserConfig(self):
        '''write the config to the file, overwrite current config file'''
        
        try:
            conf = open(PATH.CONFIG_DIR + '/' + self.currentUser + '/config', 'w')
        
            for k, v in self.user:
                if type(v) == bool:
                    conf.write(k + '=' + str(int(v)) + '\n')
                else:
                    conf.write(k + '=' + str(v) + '\n')
            
            conf.close()
        except Exception, e:
            emesenelib.common.debug('exception writing config:\n')
            emesenelib.common.debug(e)
        
    # --- PLUGINS CONFIG --- work in progress - #
    def readPluginConfig(self, pluginName):
        '''read the plugin config file and create a dict with key and value
        of all the key=value\n in the config file'''
        
        self.pluginsConfigDict[pluginName] = {}
        conf = None
        try:
            conf = open(self.getUserConfigPath() + os.sep + pluginName + '.conf', 'r')
            string = conf.read()
                        
            for i in string.splitlines():
                if i != '':
                    try:
                        delim = i.find('=')
                        key = i[:delim]
                        value = i[delim+1:]
                        self.pluginsConfigDict[pluginName][key] = value
                    except Exception, e:
                        emesenelib.common.debug(key + ' config value is incorrect')
                    
            conf.close()
        except:
            if conf:
                conf.close()
                
    def writePluginConfig(self, pluginName):
        '''write the config to the file, overwrite current plugin config file'''

        try:
            conf = open(self.getUserConfigPath() + os.sep + pluginName + '.conf', 'w')
        
            for k, v in self.pluginsConfigDict[pluginName].iteritems(): # TODO
                conf.write(k + '=' + str(v) + '\n')

            conf.close()
        except Exception, e:
            emesenelib.common.debug('exception writing config:\n')
            emesenelib.common.debug(e)
    
    def setPluginValue(self, pluginName, key, value):
        self.pluginsConfigDict[pluginName][key] = str(value)
        self.writePluginConfig(pluginName) # TODO: config transactions?
        
    def getPluginValue(self, pluginName, key, default):
        if pluginName in self.pluginsConfigDict:
            if key in self.pluginsConfigDict[pluginName]:
                return self.pluginsConfigDict[pluginName][key]
       
        return default
       
    # --- OTHER STUFF --- #
    def _explorePath( self, path ):
        result = []
        for root, dirs, files in os.walk(path, True):
            if root != path: continue
            for dir in dirs:
                if not dir.startswith('.') and dir not in result:
                    result.append(dir)
            # stops walking the tree
            dirs = []
        return result
        
    def getThemes(self):
        '''this method is here because i dont know where to put it,
        it return a list of dirs in the theme directory'''
        
        return self._explorePath(PATH.THEME_HOME_PATH) + \
               self._explorePath(PATH.THEME_SYSTEM_WIDE_PATH)
        
    def getSmilieThemes(self):
        '''the same with smilie themes'''
        
        return self._explorePath(PATH.SMILIES_HOME_PATH) + \
               self._explorePath(PATH.SMILIES_SYSTEM_WIDE_PATH)
        
    def getLogPath(self):
        logPath = self.user['logPath']
        if logPath != '':
            return logPath
        else:
            return PATH.CONFIG_DIR + os.sep + self.currentUser + os.sep + 'logs'
        
    def getCachePath(self):
        return PATH.CONFIG_DIR + os.sep + self.currentUser + os.sep + 'cache'
        
    def getUserConfigPath(self):
        return PATH.CONFIG_DIR + os.sep + self.currentUser
    
    def getAvatarsCachePath(self):
        return PATH.CONFIG_DIR + os.sep + self.currentUser + os.sep + 'avatars'
        
    def getCustomEmoticonsPath(self):
        return PATH.CONFIG_DIR + os.sep + self.currentUser + os.sep + 'custom_emoticons'
        
    def getConversationThemesPath(self):
        return PATH.CONFIG_DIR + os.sep + 'conversation_themes'


class ConfigDict(dict):
    '''A dictionary that handles type conversion, fallbacks to default, config
    file writing, etc'''

    def __init__(self, config, writeFunc, defaults, store=None):
        self.config = config
        self.writeFunc = writeFunc
        self.defaults = defaults

        if store == None:
            store = defaults.copy()
        
        self.store = {}
        for key, value in store.iteritems():
            if type(value) == bool:
                self.store[key] = str(int(value))
            else:
                self.store[key] = str(value)
         
    def __repr__(self):
        return '<ConfigDict based on ' + repr(store) + '>'

    def getDefault(self, name):
        if name in self.defaults:
            return self.defaults[name]
        else:
            return ''

    def __getitem__(self, name):
        '''returns a item from self.store. if it is not available, return that
        item from defaults
        also does some type conversion and bool checks'''

        if name in self.store:
            newType = type(self.getDefault(name))
            if newType == bool:
                if str(self.store[name]).lower() in ('true', 'false'):
                    # workaround for some buggy configs..
                    boolValue = (str(self.store[name]).lower() == 'true')
                    self.store[name] = str(int(boolValue))
                    return boolValue
                else:
                    try:
                        # >>> int('0') == True
                        return bool(int(self.store[name]))
                    except ValueError:
                        return self.getDefault(name)
            else:
                try:
                    return newType(self.store[name])
                except ValueError:
                    return self.getDefault(name)
        else:
            emesenelib.common.debug('Config value was not found: ' + str(name))
            return self.getDefault(name)

    def __setitem__(self, name, value):
        '''sets a config key "name" to the string "value", emits
        callbacks and writes config'''

        if type(value) == bool:
            value = str(int(value))
        else:
            value = str(value)

        oldValue = None
        if name in self.store:
            oldValue = self.store[name]

        self.store[name] = value

        self.config.emit('change::' + name, value, oldValue)
        self.writeFunc() # TODO: config transactions?

    def __iter__(self):
        '''for key, value in configDict'''
        return self.store.iteritems()


def _mkbase(check, make, message, *components):
    '''base function for _mkdir and _mkfile'''
    path = os.path.join(*components)
    if not check(path):
        make(path)
        if message:
            emesenelib.common.debug(message + ' created: ' + path)
        return True
    return False

def _mkdir(message, *components):
    '''makes a directory and prints a message'''
    return _mkbase(os.path.isdir, os.mkdir, message, *components)

def _mkfile(message, *components):
    '''checks if the file exists and prints a message'''
    return _mkbase(os.path.isfile, lambda x: None, message, *components)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a module that define an abstract MainMenu'''
import gettext

_ = gettext.gettext

import Menu
import stock
import status

class MainMenu(Menu.Menu):
    '''a class that has all the fields of of the main menu'''

    def __init__(self, dialog, contacts, groups, contact_list):
        '''class contructor'''
        Menu.Menu.__init__(self)
        
        self.dialog = dialog
        self.contacts = contacts
        self.groups = groups
        self.contact_list = contact_list

        self.signal_add('quit-selected', 0)
        self.signal_add('disconnect-selected', 0)
        self.signal_add('prefereces-selected', 0)
        self.signal_add('plugins-selected', 0)
        self.signal_add('about-selected', 0)

        self._build(True)

    def _build(self, logged_in):
        '''clean and build the root menu'''
        for item in self:
            self.remove(item)

        self.append(self._build_file(logged_in))
        
        if logged_in:
            self.append(self._build_view(logged_in))
            self.append(self._build_actions(logged_in))

        self.append(self._build_options(logged_in))
        self.append(self._build_help(logged_in))

    def _build_file(self, logged_in):
        '''build and return the file menu'''
        self.file_menu = Menu.Menu('_File')

        # file menu
        self.quit_item = Menu.Item(_('_Quit'), Menu.Item.TYPE_STOCK, 
            stock.QUIT, Menu.Accel('Q'))
        self.quit_item.signal_connect('selected', self._on_quit_selected)

        if logged_in:
            self.status_item = Menu.Menu(_('_Status'))
            for stat in status.ORDERED:
                if stat != status.OFFLINE:
                    temp_item = Menu.Item(status.STATUS[stat])
                    temp_item.signal_connect('selected', 
                        self._on_status_selected, stat)
                    self.status_item.append(temp_item)
        
            self.disconnect_item = Menu.Item(_('_Disconnect'), 
                Menu.Item.TYPE_STOCK, stock.DISCONNECT)
            self.disconnect_item.signal_connect('selected', 
                self._on_disconnect_selected)

            self.file_menu.append(self.status_item)
            self.file_menu.append(Menu.Item('-'))
            self.file_menu.append(self.disconnect_item)

        self.file_menu.append(self.quit_item)

        return self.file_menu

    def _build_view(self, logged_in):
        '''build and return the view menu'''
        self.view_menu = Menu.Menu(_('_View'))

        self.order_option = Menu.Option()
        self.by_status_option = Menu.Radio(_('Order by _status'))
        self.by_status_option.signal_connect('toggled', 
            self._on_by_status_toggled)
        self.by_group_option = Menu.Radio(_('Order by _group'))
        self.by_group_option.signal_connect('toggled', 
            self._on_by_group_toggled)

        self.order_option.append(self.by_status_option)
        self.order_option.append(self.by_group_option)

        self.show_by_nick_option = Menu.CheckBox(_('Show by _nick'))
        self.show_by_nick_option.signal_connect('toggled', 
            self._on_by_nick_toggled)

        self.show_offline_option = Menu.CheckBox(_('Show _offline'))
        self.show_offline_option.signal_connect('toggled', 
            self._on_show_offline_toggled)

        self.show_empty_groups_option = Menu.CheckBox(_('Show _empty groups'))
        self.show_empty_groups_option.signal_connect('toggled', 
            self._on_empty_groups_toggled)

        self.view_menu.append(self.order_option)
        self.view_menu.append(Menu.Item('-'))
        self.view_menu.append(self.show_by_nick_option)
        self.view_menu.append(self.show_offline_option)
        self.view_menu.append(self.show_empty_groups_option)

        return self.view_menu
        
    def _build_actions(self, logged_in):
        '''build and return the actions menu'''
        self.actions_menu = Menu.Menu(_('_Actions'))
        self.contact_actions_menu = Menu.Menu(_('_Contact'))
        self.group_actions_menu = Menu.Menu('_Group')

        # group actions
        self.add_group_item = Menu.Item(_('_Add'), Menu.Item.TYPE_STOCK, 
            stock.ADD)
        self.add_group_item.signal_connect('selected', 
            self._on_add_group_selected)
        
        self.remove_group_item = Menu.Item(_('_Remove'), Menu.Item.TYPE_STOCK, 
            stock.REMOVE)
        self.remove_group_item.signal_connect('selected', 
            self._on_remove_group_selected)
        
        self.rename_group_item = Menu.Item(_('Re_name'), Menu.Item.TYPE_STOCK, 
            stock.EDIT)
        self.rename_group_item.signal_connect('selected', 
            self._on_rename_group_selected)
        
        # contact actions
        self.add_contact_item = Menu.Item(_('_Add'), Menu.Item.TYPE_STOCK, 
            stock.ADD)
        self.add_contact_item.signal_connect('selected', 
            self._on_add_contact_selected)
        
        self.remove_item = Menu.Item(_('_Remove'), Menu.Item.TYPE_STOCK, 
            stock.REMOVE)
        self.remove_item.signal_connect('selected', self._on_remove_selected)
        
        self.block_item = Menu.Item(_('_Block'), Menu.Item.TYPE_STOCK, 
            stock.STOP)
        self.block_item.signal_connect('selected', self._on_block_selected)
        
        self.unblock_item = Menu.Item(_('_Unblock'), Menu.Item.TYPE_STOCK, 
            stock.APPLY)
        self.unblock_item.signal_connect('selected', self._on_unblock_selected)
        
        self.move_item = Menu.Item(_('_Move'), Menu.Item.TYPE_STOCK, 
            stock.FORWARD)
        self.move_item.signal_connect('selected', self._on_move_selected)
        
        self.set_alias_item = Menu.Item(_('Set _alias'), Menu.Item.TYPE_STOCK, 
            stock.EDIT)
        self.set_alias_item.signal_connect('selected', 
            self._on_set_alias_selected)

        # account actions
        self.set_nick_item = Menu.Item(_('Set _nick'), Menu.Item.TYPE_STOCK, 
            stock.EDIT)
        self.set_nick_item.signal_connect('selected', 
            self._on_set_nick_selected)
        
        self.set_message_item = Menu.Item(_('Set _message'), 
            Menu.Item.TYPE_STOCK, stock.EDIT)
        self.set_message_item.signal_connect('selected', 
            self._on_set_message_selected)
        
        self.set_picture_item = Menu.Item(_('Set _picture'), 
            Menu.Item.TYPE_STOCK, stock.EDIT)
        self.set_picture_item.signal_connect('selected', 
            self._on_set_picture_selected)
        
        self.contact_actions_menu.append(self.add_contact_item)
        self.contact_actions_menu.append(self.remove_item)
        self.contact_actions_menu.append(self.block_item)
        self.contact_actions_menu.append(self.unblock_item)
        self.contact_actions_menu.append(self.set_alias_item)
        self.contact_actions_menu.append(self.move_item)

        self.group_actions_menu.append(self.add_group_item)
        self.group_actions_menu.append(self.remove_group_item)
        self.group_actions_menu.append(self.rename_group_item)

        self.actions_menu.append(self.contact_actions_menu)
        self.actions_menu.append(self.group_actions_menu)
        self.actions_menu.append(self.set_nick_item)
        self.actions_menu.append(self.set_message_item)
        self.actions_menu.append(self.set_picture_item)

        return self.actions_menu
        
    def _build_options(self, logged_in):
        '''build and return the option menu'''
        self.option_menu = Menu.Menu(_('_Options'))
        self.prefs_item = Menu.Item(_('_Preferences'), Menu.Item.TYPE_STOCK, 
            stock.PREFERENCES, Menu.Accel('P'))
        self.prefs_item.signal_connect('selected', self._on_preferences_selected)

        if logged_in:
            self.plugins_item = Menu.Item(_('Plug_ins'), Menu.Item.TYPE_STOCK, 
                stock.DISCONNECT)
            self.plugins_item.signal_connect('selected', 
                self._on_plugins_selected)
            self.option_menu.append(self.plugins_item)
        
        self.option_menu.append(self.prefs_item)

        return self.option_menu
        
    def _build_help(self, logged_in):
        '''build and return the help menu'''
        self.help_menu = Menu.Menu(_('_Help'))
        self.about_item = Menu.Item(_('_About'), Menu.Item.TYPE_STOCK, 
            stock.ABOUT)

        self.help_menu.append(self.about_item)

        return self.help_menu

    # callbacks

    def _on_quit_selected(self, item):
        '''called when quit is selected'''
        self.signal_emit('quit-selected')

    def _on_disconnect_selected(self, item):
        '''called when disconnect is selected'''
        self.signal_emit('disconnect-selected')

    def _on_status_selected(self, item, stat):
        '''called when a status is selected'''
        self.contacts.set_status(stat)

    def _on_by_status_toggled(self, option, value):
        '''called when order by status is toggled'''
        if value:
            self.contact_list.order_by_status = value
    
    def _on_by_group_toggled(self, option, value):
        '''called when order by group is toggled'''
        if value:
            self.contact_list.order_by_group = value

    def _on_by_nick_toggled(self, check, value):
        '''called when show by nick is toggled'''
        self.contact_list.show_by_nick = value
    
    def _on_show_offline_toggled(self, check, value):
        '''called when show offline is toggled'''
        self.contact_list.show_offline = value
    
    def _on_empty_groups_toggled(self, check, value):
        '''called when show empty groups is toggled'''
        self.contact_list.show_empty_groups = value
    
    def _on_preferences_selected(self, item):
        '''called when preferences is selected'''
        self.signal_emit('preferences-selected')

    def _on_plugins_selected(self, item):
        '''called when plugins is selected'''
        self.signal_emit('plugins-selected')

    def _on_about_selected(self, item):
        '''called when about is selected'''
        self.signal_emit('about-selected')

    def _on_add_group_selected(self, item):
        '''called when add_group is selected'''
        self.groups.add_dialog()

    def _on_remove_group_selected(self, item):
        '''called when remove_group is selected'''
        group = self.contact_list.get_group_selected()

        def _yes_no_cb(response):
            '''callback from the confirmation dialog'''
            if response == stock.YES:
                self.groups.remove(group.name)

        if group:
            self.dialog.yes_no(
                _('Are you sure you want to remove group %s?') % \
                (group.name,), _yes_no_cb)
        else:
            self.dialog.error(_('No group selected'))

    def _on_rename_group_selected(self, item):
        '''called when rename_group is selected'''
        group = self.contact_list.get_group_selected()
        
        if group:
            self.groups.rename_dialog(group.name)
        else:
            self.dialog.error(_('No group selected'))

    def _on_add_contact_selected(self, item):
        '''called when add is selected'''
        self.contacts.add_dialog()

    def _on_remove_selected(self, item):
        '''called when remove is selected'''
        contact = self.contact_list.get_contact_selected()

        def _yes_no_cb(response):
            '''callback from the confirmation dialog'''
            if response == stock.YES:
                self.contacts.remove(contact.account)

        if contact:
            self.dialog.yes_no(
                _('Are you sure you want to remove contact %s?') % \
                (contact.account,), _yes_no_cb)
        else:
            self.dialog.error(_('No contact selected'))

    def _on_block_selected(self, item):
        '''called when block is selected'''
        contact = self.contact_list.get_contact_selected()
        
        if contact:
            self.contacts.block(contact.account)
        else:
            self.dialog.error(_('No contact selected'))

    def _on_unblock_selected(self, item):
        '''called when unblock is selected'''
        contact = self.contact_list.get_contact_selected()
        
        if contact:
            self.contacts.unblock(contact.account)
        else:
            self.dialog.error(_('No contact selected'))

    def _on_move_selected(self, item):
        '''called when move is selected'''
        pass

    def _on_set_alias_selected(self, item):
        '''called when set_alias is selected'''
        contact = self.contact_list.get_contact_selected()
        
        if contact:
            self.contacts.set_alias_dialog(contact.account)
        else:
            self.dialog.error(_('No contact selected'))

    def _on_set_nick_selected(self, item):
        '''called when set nick is selected'''
        self.contacts.set_nick_dialog()

    def _on_set_message_selected(self, item):
        '''called when set message is selected'''
        self.contacts.set_message_dialog(\
            self.contacts.me.message)

    def _on_set_picture_selected(self, item):
        '''called when set picture is selected'''
        self.contacts.set_picture_dialog()




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#   This is a reduced version of Audioscrobbler Plugin (from exaile).
#
#   Thanks to "Andy Theyers <andy@isotoma.com>" 
#

import Plugin
import urllib
import datetime
import md5
import re
import time 
import urllib2
import socket

class AudioScrobbler:
    
    """
    Provide the ability to post tracks played to a user's Last.fm
    account
    """
    
    def __init__(self,username=u'',password=u''):
        
        # Capture the information passed for future use
        self.params = dict(username=username,
                           password=password,
                           client_name=u'tst',
                           client_version=u'1.0',
                           protocol_version=u'1.1',
                           host=u'post.audioscrobbler.com')
        
        self.auth_details = {}
        self.last_shake = None
        self.authenticated = False
        self.posturl = None
        socket.setdefaulttimeout(2)


    def updatecfg(self,username=u'',password=u''):
        self.params['username'] = username
        self.params['password'] = password
        self.authenticated = False

    def auth(self):
        
        """ Authenticate against the server """
        
        if self.authenticated:
            return True
        
        now = datetime.datetime.utcnow()
        interval = datetime.timedelta(minutes=30)
        if self.last_shake is not None and self.last_shake + interval > now:
            last_shake_string = self.last_shake.strftime("%Y-%m-%d %H:%M:%S")
            print "Tried to reauthenticate too soon after last try (last try at %s)" % last_shake_string
            return False

        p = {}
        p['hs'] = 'true'
        p['u'] = self.params['username']
        p['c'] = self.params['client_name']
        p['v'] = self.params['client_version']
        p['p'] = self.params['protocol_version']
        plist = [(k, urllib.quote_plus(v.encode('utf8'))) for k, v in p.items()]
        
        authparams = urllib.urlencode(plist)
        url = 'http://%s/?%s' % (self.params['host'], authparams)
        
        req = urllib2.Request(url)
        try:
            url_handle = urllib2.urlopen(req)
        except:
            self.authenticated = False
            print "Connection Error: %s" % req
            return False

        self.last_shake = datetime.datetime.utcnow()
        response = url_handle.readlines()
        
        if len(response) == 0:
                print "Connection Error: no response" 
        
        # Get the interval if there is one
        find_interval = re.match('INTERVAL (\d+)', response[-1])
        
        username = self.params['username']
        password = self.params['password']
        # First we test the best and most likely case
        if response[0].startswith('UPTODATE'):
            ask = response[1].strip()
            answer = md5.md5(md5.md5(password).hexdigest() + ask).hexdigest()
            self.auth_details['u'] = urllib.quote_plus(username.encode('utf8'))
            self.auth_details['s'] = answer
            self.posturl = response[2].strip()
            self.authenticated = True
            
        # Next we test the least significant failure.
        elif response[0].startswith('UPDATE'):
            ask = response[1].strip()
            answer = md5.md5(md5.md5(password).hexdigest() + ask).hexdigest()
            
            self.auth_details['u'] = urllib.quote_plus(username.encode('utf8'))
            self.auth_details['s'] = answer
            self.posturl = response[2].strip()
            self.authenticated = True
            
        # Then the various failure states....
        elif response[0].startswith('BADUSER'):
            self.authenticated = False
            print "Error Username '%s' unknown by Last.fm" % username
            return False

        elif response[0].startswith('FAILED'):
            self.authenticated = False
            reason = response[0][6:]
            print "Error %s" % reason
            return False

        else:
            self.authenticated = False
            print "Unknown response from server: %r" % (response,)
            return False
        
        return True
    
    def post(self, 
                  artist_name,
                  song_title,
                  sane_length,
                  date_played, 
                  album=u'',
                  mbid=u''):
        
        track = {'a[%s]': artist_name.decode('utf8', 'replace').encode('utf8'),
                't[%s]': song_title.decode('utf8', 'replace').encode('utf8'),
                'l[%s]': str(sane_length),
                'i[%s]': date_played,
                'b[%s]': album.decode('utf8', 'replace').encode('utf8'),
                'm[%s]': mbid.encode('utf8'),
                }
               
        params = {}
        count = 0
        for k in track.keys():
            track[k] = urllib.quote(track[k].encode('utf-8'))
            params[k % (count,)] = track[k]
        
        if not self.auth():
            return False
        
        params.update(self.auth_details)
        
        postdata = '&'.join(['%s=%s' % (k, v) for k, v in params.iteritems()])
        
        try:
            req = urllib2.Request(url=self.posturl, data=postdata)
            url_handle = urllib2.urlopen(req)
            response = url_handle.readlines()
        except:
            print 'Connection Error %s' % req
            return False
        
        if len(response) == 0:
            return False
        
        # Test the various responses possibilities:
        if response[0].startswith('OK'):
            return True
        elif response[0].startswith('BADAUTH'):
            print 'Connection Error: Got BADAUTH'
            self.authenticated = False
            return False
        elif response[0].startswith('FAILED'):
            reason = response[0][6:].strip()
            print 'Connection Error %s' % reason
            return False
        else:
            print "Connection Error: Server returned something unexpected"
            return False

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''

    def __init__( self, controller, msn ):
        '''Constructor'''

        Plugin.Plugin.__init__( self, controller, msn)

        self.description = _( 'Send information to last.fm' )
        self.authors = { 'Mattia \'matz\' Pini' : 'matz.mz at gmail dot com' }
        self.website = ''
        self.displayName = _( 'Last.fm' )
        self.name = 'lastfm'

        self.config = controller.config
        self.config.readPluginConfig(self.name)

        self.controller = controller
        
        self.password = self.config.getPluginValue( self.name, 'pass', '' )
        self.username = self.config.getPluginValue( self.name, 'user', '' )

        self.lastfm =  AudioScrobbler(self.username,self.password)
        
        self.cache = []

    def on_currentmediaChange(self,msnp,user,cm, dict):
        if dict == None:
            return

        lt = time.gmtime(time.time())
        date = "%02d-%02d-%02d %02d:%02d:%02d" % (lt[0], lt[1], lt[2],lt[3], lt[4], lt[5])
        
        if dict['artist'] != '' or dict['title'] != '' or dict['album'] != '':
            self.cache.append((dict['artist'],dict['title'],200,date,dict['album']))
        
        for element in self.cache[:]:
            print "Posting %s" % str(element)
            
            if self.lastfm.post(*element):
                self.cache.remove(element)
            else:
                break
                
    def start( self ):
        self.scmcId = self.controller.msn.connect( 'self-current-media-changed', self.on_currentmediaChange)
        self.enabled = True 
    
    def stop( self ):
        self.controller.msn.disconnect( self.scmcId )
        self.enabled = False

    def check( self ):
            return ( True, 'Ok' )

    def configure( self ):
        conf = []
        conf.append( Plugin.Option( 'user', str, _('Username:'), '',
            self.config.getPluginValue( self.name, 'user', self.username )) )
        conf.append( Plugin.Option( 'pass', 'passwd', _('Password:'), '',
            self.config.getPluginValue( self.name, 'pass', self.password )) )
        configWindow = Plugin.ConfigWindow( 'Last.fm', conf )
        
        response = configWindow.run()
        if response != None:
            if response.has_key( 'user' ):
                self.username =  response[ 'user' ].value
                self.config.setPluginValue( self.name, 'user', self.username )
            if response.has_key( 'pass' ):
                self.password = response[ 'pass' ].value
                self.config.setPluginValue( self.name, 'pass', self.password )
            
            self.lastfm.updatecfg(self.username,self.password)
        return True




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a module that map the abstract.stock module to gtk'''
import gtk

from abstract.stock import *

MAP = {}
MAP[ACCEPT] = gtk.STOCK_OK
MAP[ADD] = gtk.STOCK_ADD
MAP[APPLY] = gtk.STOCK_APPLY
MAP[BACK] = gtk.STOCK_GO_BACK
MAP[BOLD] = gtk.STOCK_BOLD
MAP[CANCEL] = gtk.STOCK_CANCEL
MAP[CLEAR] = gtk.STOCK_CLEAR
MAP[CLOSE] = gtk.STOCK_CLOSE
MAP[CONNECT] = gtk.STOCK_CONNECT
MAP[DELETE] = gtk.STOCK_DELETE
MAP[DISCONNECT] = gtk.STOCK_DISCONNECT
MAP[DOWN] = gtk.STOCK_GO_DOWN
MAP[EDIT] = gtk.STOCK_EDIT
MAP[ERROR] = gtk.STOCK_DIALOG_ERROR
MAP[FORWARD] = gtk.STOCK_GO_FORWARD
MAP[INFORMATION] = gtk.STOCK_DIALOG_INFO
MAP[ITALIC] = gtk.STOCK_ITALIC
MAP[NEW] = gtk.STOCK_NEW
MAP[NO] = gtk.STOCK_NO
MAP[OK] = gtk.STOCK_OK
MAP[OPEN] = gtk.STOCK_OPEN
MAP[PREFERENCES] = gtk.STOCK_PREFERENCES
MAP[PROPERTIES] = gtk.STOCK_PROPERTIES
MAP[QUESTION] = gtk.STOCK_DIALOG_QUESTION
MAP[QUIT] = gtk.STOCK_QUIT
MAP[REFRESH] = gtk.STOCK_REFRESH
MAP[REMOVE] = gtk.STOCK_REMOVE
MAP[SAVE] = gtk.STOCK_SAVE
MAP[SELECT_COLOR] = gtk.STOCK_SELECT_COLOR
MAP[SELECT_FONT] = gtk.STOCK_SELECT_FONT
MAP[STOP] = gtk.STOCK_STOP
MAP[STRIKE] = gtk.STOCK_STRIKETHROUGH
MAP[UNDERLINE] = gtk.STOCK_UNDERLINE
MAP[UP] = gtk.STOCK_GO_UP
MAP[WARNING] = gtk.STOCK_DIALOG_WARNING
MAP[YES] = gtk.STOCK_YES
MAP[ABOUT] = gtk.STOCK_ABOUT
MAP[COPY] = gtk.STOCK_COPY

def map_stock(stock_id, default=None):
    '''try to return a gtk.STOCK_ from the given stock_id
    if not found, return default'''

    if stock_id in MAP:
        return MAP[stock_id]

    return default
